------------------------------------------------------------------------------
--                                                                          --
--                               PM Bindings                                --
--                                                                          --
--                                DOS Body                                  --
--                                                                          --
--                    Binding to the OS/2 DOS API routines                  -- 
--                                                                          --
--                            $Revision: .17 $                               --
--                                                                          --
--     Copyright (c) 1994 Dimensional Media Systems, All Rights Reserved    --
--                                                                          --
--   The PM bindings are free software; you can redistribute them and/or    --
--   modify them under terms of the GNU General Public License as published --
--   by the Free Software Foundation; either version 2, or (at your         --
--   option) any later version.  The PM bindings are distributed in the     --
--   hope that they will be useful, but WITH OUT ANY WARRANTY; without even --
--   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR    --
--   PURPOSE.  See the GNU General Public License for more details.  You    --
--   should have received a copy of the GNU General Public License          --
--   distributed with The PM bindings; see file COPYING.  If not, write to  --
--   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
--   For more information about these PM bindings and their usage with GNAT --
--   you can contact Bill Yow at                                            --
--                                                                          --  
--      Dimensional Media Systems (DMS)                                     --
--      1522 Festival Dr.                                                   --
--      Houston TX, 77062                                                   --
--      Phone - (713) 486-6116                                              --
--      Email - Byow@mci.com                                                --
--                                                                          --
------------------------------------------------------------------------------

with Pm_Types;
with System;

package body Dos is

   Exec_Flag_Values : constant array (Exec_Flag_Type) of Pm_Types.U_Long := (
      Exec_Sync             => 0,
      Exec_Async            => 1,
      Exec_Async_Result     => 2,
      Exec_Trace            => 3,
      Exec_Background       => 4,
      Exec_Load             => 5,
      Exec_Async_Result_DB  => 6);


  function Exec_Program (
             Object_Buffer : Object_Buffer_Type;
             Exec_Flag     : Exec_Flag_Type;
             Arguments     : String;
             Enviorment    : String;
             Return_Codes  : Result_Codes_Type;
             Program       : String) return Api_Return_Code is

     function DosExecPgm (
         Object_Buffer   : System.Address;
         Buffer_Size     : Pm_Types.Long;
         Exec_Flag       : Pm_Types.U_Long;
         Arguments       : System.Address;
         Enviorment      : System.Address;
         Return_Codes    : System.Address;
         Program         : System.Address) return Api_Return_Code;

         pragma Import (Convention => C,
                        Entity     => DosExecPgm,
                        Link_Name  => "_DosExecPgm");

    Args : String (Arguments'first .. Arguments'last + 1);
    Env  : String (Enviorment'first .. Enviorment'last + 1);
    Pgm  : String (Program'first .. Program'last + 1);

  begin

    Args := Arguments & ASCII.Nul;
    Env  := Enviorment & ASCII.Nul;
    Pgm  := Program & ASCII.Nul;

    return DosExecPgm (
            Object_Buffer   => Object_Buffer (Object_Buffer'first)'address,
            Buffer_Size     => Pm_Types.Long (Object_Buffer'length),
            Exec_Flag       => Exec_Flag_Values (Exec_Flag),
            Arguments       => Args (Args'first)'address,
            Enviorment      => Env (Env'first)'address,
            Return_Codes    => Return_Codes'address,
            Program         => Pgm (Pgm'first)'address);

   end Exec_Program;

------------------------------------------------------------------------------


     -- Elaborate functions to create and operate on unnamed pipes

   procedure Create_Pipe (Pipe_Size    : in     Positive;
                          Write_Handle :    out Pipe_Handle;
                          Read_Handle  :    out Pipe_Handle;
                          Return_Code  :    out API_Return_Code) is

      function DosCreatePipe (Read_Handle  : System.Address;
                              Write_Handle : System.Address;
                              Pipe_Size    : Positive)
        return API_Return_Code;
      pragma Import (Convention => C,
                     Entity     => DosCreatePipe,
                     Link_Name  => "_DosCreatePipe");

   begin

      Return_Code := DosCreatePipe (Read_Handle  => Read_Handle'ADDRESS,
                                    Write_Handle => Write_Handle'ADDRESS,
                                    Pipe_Size    => Pipe_Size);

   end Create_Pipe;

   procedure Duplicate_Handle (Old_File    : in     File_Handle;
                               New_File    : in out File_Handle;
                               Return_Code :    out API_Return_Code) is

      function DosDupHandle (Old_File_Handle : File_Handle;
                             New_File_Handle : System.Address)
        return API_Return_Code;
      pragma Import (Convention => C,
                     Entity     => DosDupHandle,
                     Link_Name  => "_DosDupHandle");

   begin

      Return_Code := DosDupHandle (Old_File_Handle => Old_File,
                                   New_File_Handle => New_File'ADDRESS);

   end Duplicate_Handle;


   procedure Open (File_Name           : in     String;
                   File_Size           : in     Positive;
                   File_Attributes     : in     Pm_Types.U_Long;
                   Open_Flag           : in     Pm_Types.U_Long;
                   Open_Mode           : in     Pm_Types.U_Long;
                   Extended_Attributes : in     Extended_Attributes_Buffer;
                   File                :    out File_Handle;
                   Action_Taken        :    out PM_Types.U_Long;
                   Return_Code         :    out API_Return_Code) is
   begin

      null;

   end Open;


   Procedure Close (File        : in     File_Handle;
                    Return_Code :    out API_Return_Code) is

      function DosClose (File : File_Handle) return API_Return_Code;
      pragma Import (Convention => C,
                     Entity     => DosClose,
                     Link_Name  => "_DosClose");

   begin

      Return_Code := DosClose (File);

   end Close;

   procedure Reset_Buffer (File        : in     File_Handle;
                           Return_Code :    out API_Return_Code) is

      function DosResetBuffer (File : File_Handle) return API_Return_Code;
      pragma Import (Convention => C,
                     Entity     => DosResetBuffer,
                     Link_Name  => "_DosResetBuffer");

   begin

      Return_Code := DosResetBuffer (File);

   end Reset_Buffer;

   procedure Read (File        : in     File_Handle;
                   Data        :    out Data_Structure;
                   Bytes_Read  :    out Natural;
                   Return_Code :    out API_Return_Code) is

      function DosRead (File_Handle   : File_Handle;
                        Buffer_Area   : System.Address;
                        Buffer_Length : Natural;
                        Bytes_Read    : System.Address)
        return API_Return_Code;
      pragma Import (Convention => C,
                     Entity     => DosRead,
                     Link_Name  => "_DosRead");

   begin

      Return_Code := DosRead (File_Handle   => File,
                              Buffer_Area   => Data'ADDRESS,
                              Buffer_Length => Data'SIZE/8,
                              Bytes_Read     => Bytes_Read'ADDRESS);

   end Read;

   procedure Write (File          : in     File_Handle;
                    Data          : in     Data_Structure;
                    Bytes_Written :    out Natural;
                    Return_Code   :    out API_Return_Code) is

      function DosWrite (File_Handle   : File_Handle;
                         Buffer_Area   : System.Address;
                         Buffer_Length : Natural;
                         Bytes_Written : System.Address)
        return API_Return_Code;
      pragma Import (Convention => C,
                     Entity     => DosWrite,
                     Link_Name  => "_DosWrite");

   begin

      Return_Code := DosWrite (File_Handle   => File,
                               Buffer_Area   => Data'ADDRESS,
                               Buffer_Length => Data'SIZE/8,
                               Bytes_Written => Bytes_Written'ADDRESS);

   end Write;


 end Dos;