------------------------------------------------------------------------------
--                                                                          --
--                               PM Bindings                                --
--                                                                          --
--                                DOS Spec                                  --
--                                                                          --
--                    Binding to the OS/2 DOS API routines                  -- 
--                                                                          --
--                            $Revision: .17$                               --
--   .15
--   Exec_Program
--
--   .17
--
--   Create_Pipe
--   Duplicate_Handle
--   Open
--   Close
--   Reset_Buffer
--                                                                          --
--   Copyright (c) 1994,1995 Dimensional Media Systems, All Rights Reserved --
--                                                                          --
--   The PM bindings are free software; you can redistribute them and/or    --
--   modify them under terms of the GNU General Public License as published --
--   by the Free Software Foundation; either version 2, or (at your         --
--   option) any later version.  The PM bindings are distributed in the     --
--   hope that they will be useful, but WITH OUT ANY WARRANTY; without even --
--   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR    --
--   PURPOSE.  See the GNU General Public License for more details.  You    --
--   should have received a copy of the GNU General Public License          --
--   distributed with The PM bindings; see file COPYING.  If not, write to  --
--   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
--   For more information about these PM bindings and their usage with GNAT --
--   you can contact Bill Yow at                                            --
--                                                                          --  
--      Dimensional Media Systems (DMS)                                     --
--      1522 Festival Dr.                                                   --
--      Houston TX, 77062                                                   --
--      Phone - (713) 486-6116                                              --
--      Email - Byow@mci.com                                                --
--                                                                          --
------------------------------------------------------------------------------

with Pm_Types;

package Dos is

  
  type Result_Codes_Type is
    record
      Dos_Terminate : Pm_Types.U_Long;
      Result        : Pm_Types.U_Long;
    end record;


  type Object_Buffer_Type is array (Pm_Types.U_Short range <>) of Pm_Types.Byte;

  type Exec_Flag_Type is (
      Exec_Sync,
      Exec_Async,
      Exec_Async_Result,
      Exec_Trace,
      Exec_Background,
      Exec_Load,
      Exec_Async_Result_DB);

  
  type Api_Return_Code is new PM_Types.U_Long;

  function Exec_Program (
             Object_Buffer : Object_Buffer_Type;
             Exec_Flag     : Exec_Flag_Type;
             Arguments     : String;
             Enviorment    : String;
             Return_Codes  : Result_Codes_Type;
             Program       : String) return Api_Return_Code;

  -------------------------------------------------------------------
  -- Declare types used by pipes and files

   type File_Handle is new Pm_Types.Long;  -- PM's HFile

   subtype Pipe_Handle is File_Handle;    -- PM's HPipe

   type Extended_Attributes_Buffer is new Pm_Types.Long;


     -- Define constants commonly used in file and pipe transactions

   Empty_Handle    : constant File_Handle := -1;

   Standard_Input  : constant File_Handle := 0;
   Standard_Output : constant File_Handle := 1;


     -- Define functions to create and operate on unnamed pipes

   procedure Create_Pipe (Pipe_Size    : in     Positive;
                          Write_Handle :    out Pipe_Handle;
                          Read_Handle  :    out Pipe_Handle;
                          Return_Code  :    out API_Return_Code);

   procedure Duplicate_Handle (Old_File    : in     File_Handle;
                               New_File    : in out File_Handle;
                               Return_Code :    out API_Return_Code);

   procedure Open (File_Name           : in     String;
                   File_Size           : in     Positive;
                   File_Attributes     : in     Pm_Types.U_Long;
                   Open_Flag           : in     Pm_Types.U_Long;
                   Open_Mode           : in     Pm_Types.U_Long;
                   Extended_Attributes : in     Extended_Attributes_Buffer;
                   File                :    out File_Handle;
                   Action_Taken        :    out PM_Types.U_Long;
                   Return_Code         :    out API_Return_Code);


   procedure Close (File        : in     File_Handle;
                    Return_Code :    out API_Return_Code);

   procedure Reset_Buffer (File        : in     File_Handle;
                           Return_Code :    out API_Return_Code);


   generic

      type Data_Structure is private;

   procedure Read (File        : in     File_Handle;
                   Data        :    out Data_Structure;
                   Bytes_Read  :    out Natural;
                   Return_Code :    out API_Return_Code);

   generic

      type Data_Structure is private;

   procedure Write (File          : in     File_Handle;
                    Data          : in     Data_Structure;
                    Bytes_Written :    out Natural;
                    Return_Code   :    out API_Return_Code);

  end Dos;
              
