/* Expands front end tree to back end RTL for GNU C-Compiler
   Copyright (C) 1987, 88, 89, 91, 92, 93, 1994 Free Software Foundation, Inc.

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */


/* This file handles the generation of rtl code from tree structure
   at the level of the function as a whole.
   It creates the rtl expressions for parameters and auto variables
   and has full responsibility for allocating stack slots.

   `expand_function_start' is called at the beginning of a function,
   before the function body is parsed, and `expand_function_end' is
   called after parsing the body.

   Call `assign_stack_local' to allocate a stack slot for a local variable.
   This is usually done during the RTL generation for the function body,
   but it can also be done in the reload pass when a pseudo-register does
   not get a hard register.

   Call `put_var_into_stack' when you learn, belatedly, that a variable
   previously given a pseudo-register must in fact go in the stack.
   This function changes the DECL_RTL to be a stack slot instead of a reg
   then scans all the RTL instructions so far generated to correct them.  */

#include "config.h"

#include <stdio.h>

#include "rtl.h"
#include "tree.h"
#include "flags.h"
#include "function.h"
/* #include "insn-flags.h" */
#include "expr.h"
/* #include "insn-codes.h" */
/* #include "regs.h" */
/* #include "hard-reg-set.h" */
/* #include "insn-config.h" */
/* #include "recog.h" */
/* #include "output.h" */
/* #include "basic-block.h" */
#include "obstack.h"
/* #include "bytecode.h" */

/* Some systems use __main in a way incompatible with its use in gcc, in these
   cases use the macros NAME__MAIN to give a quoted symbol and SYMBOL__MAIN to
   give the same symbol without quotes for an alternative entry point.  You
   must define both, or niether. */
#ifndef NAME__MAIN
#define NAME__MAIN "__main"
#define SYMBOL__MAIN __main
#endif

/* Round a value to the lowest integer less than it that is a multiple of
   the required alignment.  Avoid using division in case the value is
   negative.  Assume the alignment is a power of two.  */
#define FLOOR_ROUND(VALUE,ALIGN) ((VALUE) & ~((ALIGN) - 1))

/* Similar, but round to the next highest integer that meets the
   alignment.  */
#define CEIL_ROUND(VALUE,ALIGN)	(((VALUE) + (ALIGN) - 1) & ~((ALIGN)- 1))

/* NEED_SEPARATE_AP means that we cannot derive ap from the value of fp
   during rtl generation.  If they are different register numbers, this is
   always true.  It may also be true if
   FIRST_PARM_OFFSET - STARTING_FRAME_OFFSET is not a constant during rtl
   generation.  See fix_lexical_addr for details.  */

#if ARG_POINTER_REGNUM != FRAME_POINTER_REGNUM
#define NEED_SEPARATE_AP
#endif

/* Number of bytes of args popped by function being compiled on its return.
   Zero if no bytes are to be popped.
   May affect compilation of return insn or of function epilogue.  */

int current_function_pops_args;

/* Nonzero if function being compiled needs to be given an address
   where the value should be stored.  */

int current_function_returns_struct;

/* Nonzero if function being compiled needs to
   return the address of where it has put a structure value.  */

int current_function_returns_pcc_struct;

/* Nonzero if function being compiled needs to be passed a static chain.  */

int current_function_needs_context;

/* Nonzero if function being compiled can call setjmp.  */

int current_function_calls_setjmp;

/* Nonzero if function being compiled can call longjmp.  */

int current_function_calls_longjmp;

/* Nonzero if function being compiled receives nonlocal gotos
   from nested functions.  */

int current_function_has_nonlocal_label;

/* Nonzero if function being compiled has nonlocal gotos to parent
   function.  */

int current_function_has_nonlocal_goto;

/* Nonzero if function being compiled contains nested functions.  */

int current_function_contains_functions;

/* Nonzero if function being compiled can call alloca,
   either as a subroutine or builtin.  */

int current_function_calls_alloca;

/* Nonzero if the current function returns a pointer type */

int current_function_returns_pointer;

/* If some insns can be deferred to the delay slots of the epilogue, the
   delay list for them is recorded here.  */

rtx current_function_epilogue_delay_list;

/* If function's args have a fixed size, this is that size, in bytes.
   Otherwise, it is -1.
   May affect compilation of return insn or of function epilogue.  */

int current_function_args_size;

/* # bytes the prologue should push and pretend that the caller pushed them.
   The prologue must do this, but only if parms can be passed in registers.  */

int current_function_pretend_args_size;

/* # of bytes of outgoing arguments.  If ACCUMULATE_OUTGOING_ARGS is
   defined, the needed space is pushed by the prologue. */

int current_function_outgoing_args_size;

/* This is the offset from the arg pointer to the place where the first
   anonymous arg can be found, if there is one.  */

rtx current_function_arg_offset_rtx;

/* Nonzero if current function uses varargs.h or equivalent.
   Zero for functions that use stdarg.h.  */

int current_function_varargs;

/* Quantities of various kinds of registers
   used for the current function's args.  */

CUMULATIVE_ARGS current_function_args_info;

/* Name of function now being compiled.  */

char *current_function_name;

/* If non-zero, an RTL expression for that location at which the current
   function returns its result.  Always equal to
   DECL_RTL (DECL_RESULT (current_function_decl)), but provided
   independently of the tree structures.  */

rtx current_function_return_rtx;

/* Nonzero if the current function uses the constant pool.  */

int current_function_uses_const_pool;

/* Nonzero if the current function uses pic_offset_table_rtx.  */
int current_function_uses_pic_offset_table;

/* The arg pointer hard register, or the pseudo into which it was copied.  */
rtx current_function_internal_arg_pointer;

/* The FUNCTION_DECL for an inline function currently being expanded.  */
tree inline_function_decl;

/* Number of function calls seen so far in current function.  */

int function_call_count;

/* List (chain of TREE_LIST) of LABEL_DECLs for all nonlocal labels
   (labels to which there can be nonlocal gotos from nested functions)
   in this function.  */

tree nonlocal_labels;

/* RTX for stack slot that holds the current handler for nonlocal gotos.
   Zero when function does not have nonlocal labels.  */

rtx nonlocal_goto_handler_slot;

/* RTX for stack slot that holds the stack pointer value to restore
   for a nonlocal goto.
   Zero when function does not have nonlocal labels.  */

rtx nonlocal_goto_stack_level;

/* Label that will go on parm cleanup code, if any.
   Jumping to this label runs cleanup code for parameters, if
   such code must be run.  Following this code is the logical return label.  */

rtx cleanup_label;

/* Label that will go on function epilogue.
   Jumping to this label serves as a "return" instruction
   on machines which require execution of the epilogue on all returns.  */

rtx return_label;

/* List (chain of EXPR_LISTs) of pseudo-regs of SAVE_EXPRs.
   So we can mark them all live at the end of the function, if nonopt.  */
rtx save_expr_regs;

/* List (chain of EXPR_LISTs) of all stack slots in this function.
   Made for the sake of unshare_all_rtl.  */
rtx stack_slot_list;

/* Chain of all RTL_EXPRs that have insns in them.  */
tree rtl_expr_chain;

/* Label to jump back to for tail recursion, or 0 if we have
   not yet needed one for this function.  */
rtx tail_recursion_label;

/* Place after which to insert the tail_recursion_label if we need one.  */
rtx tail_recursion_reentry;

/* Location at which to save the argument pointer if it will need to be
   referenced.  There are two cases where this is done: if nonlocal gotos
   exist, or if vars stored at an offset from the argument pointer will be
   needed by inner routines.  */

rtx arg_pointer_save_area;

/* Offset to end of allocated area of stack frame.
   If stack grows down, this is the address of the last stack slot allocated.
   If stack grows up, this is the address for the next slot.  */
int frame_offset;

/* List (chain of TREE_LISTs) of static chains for containing functions.
   Each link has a FUNCTION_DECL in the TREE_PURPOSE and a reg rtx
   in an RTL_EXPR in the TREE_VALUE.  */
static tree context_display;

/* List (chain of TREE_LISTs) of trampolines for nested functions.
   The trampoline sets up the static chain and jumps to the function.
   We supply the trampoline's address when the function's address is requested.

   Each link has a FUNCTION_DECL in the TREE_PURPOSE and a reg rtx
   in an RTL_EXPR in the TREE_VALUE.  */
static tree trampoline_list;

/* Insn after which register parms and SAVE_EXPRs are born, if nonopt.  */
static rtx parm_birth_insn;

#if 0
/* Nonzero if a stack slot has been generated whose address is not
   actually valid.  It means that the generated rtl must all be scanned
   to detect and correct the invalid addresses where they occur.  */
static int invalid_stack_slot;
#endif

/* Last insn of those whose job was to put parms into their nominal homes.  */
static rtx last_parm_insn;

/* 1 + last pseudo register number used for loading a copy
   of a parameter of this function.  */
static int max_parm_reg;

/* Vector indexed by REGNO, containing location on stack in which
   to put the parm which is nominally in pseudo register REGNO,
   if we discover that that parm must go in the stack.  */
static rtx *parm_reg_stack_loc;

#if 0  /* Turned off because 0 seems to work just as well.  */
/* Cleanup lists are required for binding levels regardless of whether
   that binding level has cleanups or not.  This node serves as the
   cleanup list whenever an empty list is required.  */
static tree empty_cleanup_list;
#endif

/* Nonzero once virtual register instantiation has been done.
   assign_stack_local uses frame_pointer_rtx when this is nonzero.  */
static int virtuals_instantiated;

/* These variables hold pointers to functions to
   save and restore machine-specific data,
   in push_function_context and pop_function_context.  */
void (*save_machine_status) ();
void (*restore_machine_status) ();

/* Nonzero if we need to distinguish between the return value of this function
   and the return value of a function called by this function.  This helps
   integrate.c  */

extern int rtx_equal_function_value_matters;
extern tree sequence_rtl_expr;
extern tree bc_runtime_type_code ();
extern rtx bc_build_calldesc ();
extern char *bc_emit_trampoline ();
extern char *bc_end_function ();

/* In order to evaluate some expressions, such as function calls returning
   structures in memory, we need to temporarily allocate stack locations.
   We record each allocated temporary in the following structure.

   Associated with each temporary slot is a nesting level.  When we pop up
   one level, all temporaries associated with the previous level are freed.
   Normally, all temporaries are freed after the execution of the statement
   in which they were created.  However, if we are inside a ({...}) grouping,
   the result may be in a temporary and hence must be preserved.  If the
   result could be in a temporary, we preserve it if we can determine which
   one it is in.  If we cannot determine which temporary may contain the
   result, all temporaries are preserved.  A temporary is preserved by
   pretending it was allocated at the previous nesting level.

   Automatic variables are also assigned temporary slots, at the nesting
   level where they are defined.  They are marked a "kept" so that
   free_temp_slots will not free them.  */

struct temp_slot
{
  /* Points to next temporary slot.  */
  struct temp_slot *next;
  /* The rtx to used to reference the slot. */
  rtx slot;
  /* The rtx used to represent the address if not the address of the
     slot above.  May be an EXPR_LIST if multiple addresses exist.  */
  rtx address;
  /* The size, in units, of the slot.  */
  int size;
  /* The value of `sequence_rtl_expr' when this temporary is allocated.  */
  tree rtl_expr;
  /* Non-zero if this temporary is currently in use.  */
  char in_use;
  /* Non-zero if this temporary has its address taken.  */
  char addr_taken;
  /* Nesting level at which this slot is being used.  */
  int level;
  /* Non-zero if this should survive a call to free_temp_slots.  */
  int keep;
};

/* List of all temporaries allocated, both available and in use.  */

struct temp_slot *temp_slots;

/* Current nesting level for temporaries.  */

int temp_slot_level;

/* The FUNCTION_DECL node for the current function.  */
static tree this_function_decl;

/* Callinfo pointer for the current function.  */
static rtx this_function_callinfo;

/* The label in the bytecode file of this function's actual bytecode.
   Not an rtx.  */
static char *this_function_bytecode;

/* The call description vector for the current function.  */
static rtx this_function_calldesc;

/* Size of the local variables allocated for the current function.  */
int local_vars_size;

/* Current depth of the bytecode evaluation stack.  */
int stack_depth;

/* Maximum depth of the evaluation stack in this function.  */
int max_stack_depth;

/* Current depth in statement expressions.  */
static int stmt_expr_depth;

/* This structure is used to record MEMs or pseudos used to replace VAR, any
   SUBREGs of VAR, and any MEMs containing VAR as an address.  We need to
   maintain this list in case two operands of an insn were required to match;
   in that case we must ensure we use the same replacement.  */

struct fixup_replacement
{
  rtx old;
  rtx new;
  struct fixup_replacement *next;
};
   
/* Forward declarations.  */

#if 0
static struct temp_slot *find_temp_slot_from_address  PROTO((rtx));
static void put_reg_into_stack	PROTO((struct function *, rtx, tree,
				       enum machine_mode, enum machine_mode));
static void fixup_var_refs	PROTO((rtx, enum machine_mode, int));
static struct fixup_replacement
  *find_fixup_replacement	PROTO((struct fixup_replacement **, rtx));
static void fixup_var_refs_insns PROTO((rtx, enum machine_mode, int,
					rtx, int));
static void fixup_var_refs_1	PROTO((rtx, enum machine_mode, rtx *, rtx,
				       struct fixup_replacement **));
static rtx fixup_memory_subreg	PROTO((rtx, rtx, int));
static rtx walk_fixup_memory_subreg  PROTO((rtx, rtx, int));
static rtx fixup_stack_1	PROTO((rtx, rtx));
static void optimize_bit_field	PROTO((rtx, rtx, rtx *));
static void instantiate_decls	PROTO((tree, int));
static void instantiate_decls_1	PROTO((tree, int));
static void instantiate_decl	PROTO((rtx, int, int));
static int instantiate_virtual_regs_1 PROTO((rtx *, rtx, int));
static void delete_handlers	PROTO((void));
static void pad_to_arg_alignment PROTO((struct args_size *, int));
static void pad_below		PROTO((struct args_size *, enum  machine_mode,
				       tree));
static tree round_down		PROTO((tree, int));
static rtx round_trampoline_addr PROTO((rtx));
static tree blocks_nreverse	PROTO((tree));
static int all_blocks		PROTO((tree, tree *));
static int *record_insns	PROTO((rtx));
static int contains		PROTO((rtx, int *));
#endif

/* Pointer to chain of `struct function' for containing functions.  */
struct function *outer_function_chain;

/* Given a function decl for a containing function,
   return the `struct function' for it.  */

struct function *
find_function_data (decl)
     tree decl;
{
  struct function *p;
  for (p = outer_function_chain; p; p = p->next)
    if (p->decl == decl)
      return p;
  abort ();
}

/* Save the current context for compilation of a nested function.
   This is called from language-specific code.
   The caller is responsible for saving any language-specific status,
   since this function knows only about language-independent variables.  */

void
push_function_context_to (toplevel)
     int toplevel;
{
  struct function *p = (struct function *) xmalloc (sizeof (struct function));

  p->next = outer_function_chain;
  outer_function_chain = p;

  p->name = current_function_name;
  p->decl = current_function_decl;
  p->pops_args = current_function_pops_args;
  p->returns_struct = current_function_returns_struct;
  p->returns_pcc_struct = current_function_returns_pcc_struct;
  p->needs_context = current_function_needs_context;
  p->calls_setjmp = current_function_calls_setjmp;
  p->calls_longjmp = current_function_calls_longjmp;
  p->calls_alloca = current_function_calls_alloca;
  p->has_nonlocal_label = current_function_has_nonlocal_label;
  p->has_nonlocal_goto = current_function_has_nonlocal_goto;
  p->args_size = current_function_args_size;
  p->pretend_args_size = current_function_pretend_args_size;
  p->arg_offset_rtx = current_function_arg_offset_rtx;
  p->varargs = current_function_varargs;
  p->uses_const_pool = current_function_uses_const_pool;
  p->uses_pic_offset_table = current_function_uses_pic_offset_table;
  p->internal_arg_pointer = current_function_internal_arg_pointer;
  p->max_parm_reg = max_parm_reg;
  p->parm_reg_stack_loc = parm_reg_stack_loc;
  p->outgoing_args_size = current_function_outgoing_args_size;
  p->return_rtx = current_function_return_rtx;
  p->nonlocal_goto_handler_slot = nonlocal_goto_handler_slot;
  p->nonlocal_goto_stack_level = nonlocal_goto_stack_level;
  p->nonlocal_labels = nonlocal_labels;
  p->cleanup_label = cleanup_label;
  p->return_label = return_label;
  p->save_expr_regs = save_expr_regs;
  p->stack_slot_list = stack_slot_list;
  p->parm_birth_insn = parm_birth_insn;
  p->frame_offset = frame_offset;
  p->tail_recursion_label = tail_recursion_label;
  p->tail_recursion_reentry = tail_recursion_reentry;
  p->arg_pointer_save_area = arg_pointer_save_area;
  p->rtl_expr_chain = rtl_expr_chain;
  p->last_parm_insn = last_parm_insn;
  p->context_display = context_display;
  p->trampoline_list = trampoline_list;
  p->function_call_count = function_call_count;
  p->temp_slots = temp_slots;
  p->temp_slot_level = temp_slot_level;
  p->fixup_var_refs_queue = 0;
  p->epilogue_delay_list = current_function_epilogue_delay_list;

  save_tree_status (p, toplevel);
  save_storage_status (p);
#if 0
  save_emit_status (p);
#endif
  init_emit ();
#if 0
  save_expr_status (p);
#endif
  save_stmt_status (p);
#if 0
  save_varasm_status (p);
#endif

  if (save_machine_status)
    (*save_machine_status) (p);
}

void
push_function_context ()
{
  push_function_context_to (0);
}

/* Restore the last saved context, at the end of a nested function.
   This function is called from language-specific code.  */

void
pop_function_context_from (toplevel)
     int toplevel;
{
  struct function *p = outer_function_chain;

  outer_function_chain = p->next;

  current_function_name = p->name;
  current_function_decl = p->decl;
  current_function_pops_args = p->pops_args;
  current_function_returns_struct = p->returns_struct;
  current_function_returns_pcc_struct = p->returns_pcc_struct;
  current_function_needs_context = p->needs_context;
  current_function_calls_setjmp = p->calls_setjmp;
  current_function_calls_longjmp = p->calls_longjmp;
  current_function_calls_alloca = p->calls_alloca;
  current_function_has_nonlocal_label = p->has_nonlocal_label;
  current_function_has_nonlocal_goto = p->has_nonlocal_goto;
  if (! toplevel)
    current_function_contains_functions = 1;
  current_function_args_size = p->args_size;
  current_function_pretend_args_size = p->pretend_args_size;
  current_function_arg_offset_rtx = p->arg_offset_rtx;
  current_function_varargs = p->varargs;
  current_function_uses_const_pool = p->uses_const_pool;
  current_function_uses_pic_offset_table = p->uses_pic_offset_table;
  current_function_internal_arg_pointer = p->internal_arg_pointer;
  max_parm_reg = p->max_parm_reg;
  parm_reg_stack_loc = p->parm_reg_stack_loc;
  current_function_outgoing_args_size = p->outgoing_args_size;
  current_function_return_rtx = p->return_rtx;
  nonlocal_goto_handler_slot = p->nonlocal_goto_handler_slot;
  nonlocal_goto_stack_level = p->nonlocal_goto_stack_level;
  nonlocal_labels = p->nonlocal_labels;
  cleanup_label = p->cleanup_label;
  return_label = p->return_label;
  save_expr_regs = p->save_expr_regs;
  stack_slot_list = p->stack_slot_list;
  parm_birth_insn = p->parm_birth_insn;
  frame_offset = p->frame_offset;
  tail_recursion_label = p->tail_recursion_label;
  tail_recursion_reentry = p->tail_recursion_reentry;
  arg_pointer_save_area = p->arg_pointer_save_area;
  rtl_expr_chain = p->rtl_expr_chain;
  last_parm_insn = p->last_parm_insn;
  context_display = p->context_display;
  trampoline_list = p->trampoline_list;
  function_call_count = p->function_call_count;
  temp_slots = p->temp_slots;
  temp_slot_level = p->temp_slot_level;
  current_function_epilogue_delay_list = p->epilogue_delay_list;

#if 0
  restore_tree_status (p, toplevel);
  restore_storage_status (p);
  restore_expr_status (p);
  restore_emit_status (p);
  restore_stmt_status (p);
  restore_varasm_status (p);
#endif

  if (restore_machine_status)
    (*restore_machine_status) (p);

#if 0
  /* Finish doing put_var_into_stack for any of our variables
     which became addressable during the nested function.  */
  {
    struct var_refs_queue *queue = p->fixup_var_refs_queue;
    for (; queue; queue = queue->next)
      fixup_var_refs (queue->modified, queue->promoted_mode, queue->unsignedp);
  }
#endif /* 0 */

  free (p);

  /* Reset variables that have known state during rtx generation.  */
  rtx_equal_function_value_matters = 1;
  virtuals_instantiated = 0;
}

void pop_function_context ()
{
  pop_function_context_from (0);
}

/* Free all temporaries used so far.  This is normally called at the end
   of generating code for a statement.  Don't free any temporaries
   currently in use for an RTL_EXPR that hasn't yet been emitted.
   We could eventually do better than this since it can be reused while
   generating the same RTL_EXPR, but this is complex and probably not
   worthwhile.  */

void
free_temp_slots ()
{
}

/* Retroactively move an auto variable from a register to a stack slot.
   This is done when an address-reference to the variable is seen.  */

void
put_var_into_stack (decl)
     tree decl;
{
}

static void
fixup_var_refs (var, promoted_mode, unsignedp)
     rtx var;
     enum machine_mode promoted_mode;
     int unsignedp;
{
}

/* Output a USE for any register use in RTL.
   This is used with -noreg to mark the extent of lifespan
   of any registers used in a user-visible variable's DECL_RTL.  */

void
use_variable (rtl)
     rtx rtl;
{
}

/* Like use_variable except that it outputs the USEs after INSN
   instead of at the end of the insn-chain.  */

void
use_variable_after (rtl, insn)
     rtx rtl, insn;
{
}

/* Return 1 if EXP is an aggregate type (or a value with aggregate type).
   This means a type for which function calls must pass an address to the
   function or get an address back from the function.
   EXP may be a type node or an expression (whose type is tested).  */

int
aggregate_value_p (exp)
     tree exp;
{
  int i, regno, nregs;
  rtx reg;
  tree type;
  if (TREE_CODE_CLASS (TREE_CODE (exp)) == 't')
    type = exp;
  else
    type = TREE_TYPE (exp);

  if (RETURN_IN_MEMORY (type))
    return 1;
  if (flag_pcc_struct_return && AGGREGATE_TYPE_P (type))
    return 1;
#if 0
  /* Make sure we have suitable call-clobbered regs to return
     the value in; if not, we must return it in memory.  */
  reg = hard_function_value (type, 0);
  regno = REGNO (reg);
  nregs = HARD_REGNO_NREGS (regno, TYPE_MODE (type));
  for (i = 0; i < nregs; i++)
    if (! call_used_regs[regno + i])
      return 1;
  return 0;
#else
  return 1;
#endif
}

/* Walk the tree of blocks describing the binding levels within a function
   and warn about uninitialized variables.
   This is done after calling flow_analysis and before global_alloc
   clobbers the pseudo-regs to hard regs.  */

void
uninitialized_vars_warning (block)
     tree block;
{
}

/* Do the appropriate part of uninitialized_vars_warning
   but for arguments instead of local variables.  */

void
setjmp_args_warning (block)
     tree block;
{
}

/* If this function call setjmp, put all vars into the stack
   unless they were declared `register'.  */

void
setjmp_protect (block)
     tree block;
{
}

/* Like the previous function, but for args instead of local variables.  */

void
setjmp_protect_args ()
{
}

/* The functions identify_blocks and reorder_blocks provide a way to
   reorder the tree of BLOCK nodes, for optimizers that reshuffle or
   duplicate portions of the RTL code.  Call identify_blocks before
   changing the RTL, and call reorder_blocks after.  */

/* Put all this function's BLOCK nodes into a vector, and return it.
   Also store in each NOTE for the beginning or end of a block
   the index of that block in the vector.
   The arguments are TOP_BLOCK, the top-level block of the function,
   and INSNS, the insn chain of the function.  */

tree *
identify_blocks (top_block, insns)
     tree top_block;
     rtx insns;
{
  abort();
}

/* Given BLOCK_VECTOR which was returned by identify_blocks,
   and a revised instruction chain, rebuild the tree structure
   of BLOCK nodes to correspond to the new order of RTL.
   The new block tree is inserted below TOP_BLOCK.
   Returns the current top-level block.  */

tree
reorder_blocks (block_vector, top_block, insns)
     tree *block_vector;
     tree top_block;
     rtx insns;
{
  abort();
}

/* Reverse the order of elements in the chain T of blocks,
   and return the new head of the chain (old last element).  */

static tree
blocks_nreverse (t)
     tree t;
{
  register tree prev = 0, decl, next;
  for (decl = t; decl; decl = next)
    {
      next = BLOCK_CHAIN (decl);
      BLOCK_CHAIN (decl) = prev;
      prev = decl;
    }
  return prev;
}

/* Count the subblocks of BLOCK, and list them all into the vector VECTOR.
   Also clear TREE_ASM_WRITTEN in all blocks.  */

static int
all_blocks (block, vector)
     tree block;
     tree *vector;
{
  int n_blocks = 1;
  tree subblocks; 

  TREE_ASM_WRITTEN (block) = 0;
  /* Record this block.  */
  if (vector)
    vector[0] = block;

  /* Record the subblocks, and their subblocks.  */
  for (subblocks = BLOCK_SUBBLOCKS (block);
       subblocks; subblocks = BLOCK_CHAIN (subblocks))
    n_blocks += all_blocks (subblocks, vector ? vector + n_blocks : 0);

  return n_blocks;
}

/* Generate RTL for the start of the function SUBR (a FUNCTION_DECL tree node)
   and initialize static variables for generating RTL for the statements
   of the function.  */

void
init_function_start (subr, filename, line)
     tree subr;
     char *filename;
     int line;
{
  char *junk;

  init_stmt_for_function ();

  /* No stack slots have been made yet.  */
  stack_slot_list = 0;

  /* There is no stack slot for handling nonlocal gotos.  */
  nonlocal_goto_handler_slot = 0;
  nonlocal_goto_stack_level = 0;

  /* No labels have been declared for nonlocal use.  */
  nonlocal_labels = 0;

  /* No function calls so far in this function.  */
  function_call_count = 0;

  /* No parm regs have been allocated.
     (This is important for output_inline_function.)  */
  max_parm_reg = LAST_VIRTUAL_REGISTER + 1;

  /* Initialize the RTL mechanism.  */
  init_emit ();

  /* Initialize the queue of pending postincrement and postdecrements,
     and some other info in expr.c.  */
  init_expr ();

  current_function_name = (*decl_printable_name) (subr, &junk);

  /* Nonzero if this is a nested function that uses a static chain.  */

  current_function_needs_context
    = (decl_function_context (current_function_decl) != 0);

  /* Set if a call to setjmp is seen.  */
  current_function_calls_setjmp = 0;

  /* Set if a call to longjmp is seen.  */
  current_function_calls_longjmp = 0;

  current_function_calls_alloca = 0;
  current_function_has_nonlocal_label = 0;
  current_function_has_nonlocal_goto = 0;
  current_function_contains_functions = 0;

  current_function_returns_pcc_struct = 0;
  current_function_returns_struct = 0;
  current_function_epilogue_delay_list = 0;
  current_function_uses_const_pool = 0;
  current_function_uses_pic_offset_table = 0;

  /* We have not yet needed to make a label to jump to for tail-recursion.  */
  tail_recursion_label = 0;

  /* We haven't had a need to make a save area for ap yet.  */

  arg_pointer_save_area = 0;

  /* No stack slots allocated yet.  */
  frame_offset = 0;

  /* No SAVE_EXPRs in this function yet.  */
  save_expr_regs = 0;

  /* No RTL_EXPRs in this function yet.  */
  rtl_expr_chain = 0;

  /* We have not allocated any temporaries yet.  */
  temp_slots = 0;
  temp_slot_level = 0;

  /* Within function body, compute a type's size as soon it is laid out.  */
  immediate_size_expand++;

  /* We haven't made any trampolines for this function yet.  */
  trampoline_list = 0;

  /* Warn if this value is an aggregate type,
     regardless of which calling convention we are using for it.  */
  if (warn_aggregate_return
      && AGGREGATE_TYPE_P (TREE_TYPE (DECL_RESULT (subr))))
    warning ("function returns an aggregate");

  current_function_returns_pointer
    = (TREE_CODE (TREE_TYPE (DECL_RESULT (subr))) == POINTER_TYPE);

  /* Indicate that we need to distinguish between the return value of the
     present function and the return value of a function being called.  */
  rtx_equal_function_value_matters = 1;

  /* Indicate that we have not instantiated virtual registers yet.  */
  virtuals_instantiated = 0;

  /* By default assume not varargs.  */
  current_function_varargs = 0;
}

/* Indicate that the current function uses extra args
   not explicitly mentioned in the argument list in any fashion.  */

void
mark_varargs ()
{
  current_function_varargs = 1;
}

/* Expand a call to __main at the beginning of a possible main function.  */

#if defined(INIT_SECTION_ASM_OP) && !defined(INVOKE__main)
#undef HAS_INIT_SECTION
#define HAS_INIT_SECTION
#endif

void
expand_main_function ()
{
}

extern struct obstack permanent_obstack;


/* Expand end of bytecode function. See details the comment of
   expand_function_end(), below. */

void
bc_expand_function_end ()
{
}


/* Start the RTL for a new function, and set variables used for
   emitting RTL.
   SUBR is the FUNCTION_DECL node.
   PARMS_HAVE_CLEANUPS is nonzero if there are cleanups associated with
   the function's parameters, which must be run at any return statement.  */

void
expand_function_start (subr, parms_have_cleanups)
     tree subr;
     int parms_have_cleanups;
{
}

/* Generate RTL for the end of the current function.
   FILENAME and LINE are the current position in the source file. 

   It is up to language-specific callers to do cleanups for parameters--
   or else, supply 1 for END_BINDINGS and we will call expand_end_bindings.  */

void
expand_function_end (filename, line, end_bindings)
     char *filename;
     int line;
     int end_bindings;
{
}

/* Reposition the prologue-end and epilogue-begin notes after instruction
   scheduling and delayed branch scheduling.  */

void
reposition_prologue_and_epilogue_notes (f)
     rtx f;
{
}
