/************************************************************************
 *  Forest library, version 1.2						*
 *  Copyright (C) 1995 by Kenneth Almquist				*
 *									*
 *  You may redistribute or modify this software under the terms	*
 *  given in the License for the Forest Library, which appears in	*
 *  the file named LICENSE.						*
 ************************************************************************/

/*
 * This program writes various constants from C header files to the
 * file cdefs.m4.
 */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <unistd.h>
#include <limits.h>
#include <signal.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <termios.h>


struct lval {
    const char *name;
    long value;
    int mode;
};

#include "cconst.h"	/* defines longconst array */

FILE *fp;


main() {
    const struct lval *lp;
    char val[32];
    int i;
    int errflag;

    /* Open the output file. */
    if ((fp = fopen("cdefs.m4", "w")) == NULL) {
	fputs("Cannot create cdefs.m4\n", stderr);
	exit(2);
    }

    /* Write values of constants. */
    errflag = 0;
    for (lp = longconst ; lp->name ; lp++) {
	switch (lp->mode) {
	    case 0:
		sprintf(val, "[%ld]", lp->value);
		break;
	    case 1:
		sprintf(val, "[[8#%o#]]", lp->value);
		break;
	    case 2:
		for (i = 0 ; ; i++) {
		    if ((1 << i) == 0) {
			fprintf(stderr, "Bad definition for %s: 0x%lX\n",
				lp->name, lp->value);
			errflag++;
			i = 0;
			break;
		    }
		    if (lp->value == (1 << i)) {
			break;
		    }
		}
		sprintf(val, "[Element_%d]", i);
		break;
	    case 3:
		if (lp->value)
		    strcpy(val, "[true]");
		else
		    strcpy(val, "[false]");
		break;
	    case 4:
		fprintf(stderr, "%s undefined\n", lp->name);
		errflag++;
		strcpy(val, "[0]");
		break;
	}
	fprintf(fp, "define(%s, %s)\n", lp->name, val);
    }
    exit(errflag? 2 : 0);
}
