--------------------------------------------------------------------------
--  Forest library, version 1.2						--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

-- This package contains operations which are considered "unsafe" because
-- their interaction with Ada tasking is not fully defined.  As long as
-- you do not plan to use tasking, there should be no problem with these
-- primitives.

with POSIX,
     POSIX_Process_Environment,
     POSIX_Process_Identification;
package POSIX_Unsafe_Process_Primitives is

    -- Process Creation

    function Fork return POSIX_Process_Identification.Process_ID;
	-- Exceptions:  posix_error.
	-- Creates a copy of the current process.  In the parent process,
	-- returns the process_id of the child process.  In the child
	-- process, returns null_process_id.  Ideally, the child process
	-- will contain only the task which called fork, but in many
	-- implementations fork will copy all the tasks in a process.

    -- File Execution

    procedure Exec
            (Pathname:	POSIX.Pathname;
             Arg_List:	POSIX.POSIX_String_List := POSIX.Empty_String_List;
             Env_List:	POSIX_Process_Environment.Environment);
	-- Exceptions:  posix_error.
	-- Execute a new program, which replaces the current one.  Pathname
	-- specifies the program to execute.  Arg_list contains the arguments
	-- to the program; by convention the first argument should be the
	-- name of the program.  Env_list contains the environment variables
	-- to be passed to the program.  This routine never returns; it either
	-- overlays the current program with a new one or raises posix_error.

    procedure Exec
            (Pathname:	POSIX.Pathname;
             Arg_List:	POSIX.POSIX_String_List := POSIX.Empty_String_List);
	-- Exceptions:  posix_error.
	-- As above, but passes the current envrionment to the program.

    procedure Exec_Search
            (Filename:	POSIX.Filename;
             Arg_List:	POSIX.POSIX_String_List := POSIX.Empty_String_List;
             Env_List:	POSIX_Process_Environment.Environment);
	-- Exceptions:  posix_error.
	-- If filename contains one or more slashes, this is the same as exec.
	-- Otherwise, it searches for the file name using the PATH environment
	-- variable (in the current environment, not in the environment passed
	-- to the program.)

    procedure Exec_Search
            (Filename:	POSIX.Filename;
             Arg_List:	POSIX.POSIX_String_List := POSIX.Empty_String_List);
	-- Exceptions:  posix_error.
	-- As above, but passes the current envrionment to the program.

end POSIX_Unsafe_Process_Primitives;
