--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------


with ada.command_line;  use ada.command_line;
with text_io; use text_io;
with posix; use posix;
with posix_process_environment; use posix_process_environment;
with posix_io; use posix_io;
with posix_file_status; use posix_file_status;
with posix_signals; use posix_signals;
procedure child is
    error_count: natural := 0;
    good_exit_status: exit_status := 0;
    procedure fail(message: string) is
    begin
	put("child: Test failed: "); put(message); new_line;
	error_count := error_count + 1;
    end fail;
    blocked: signal_set := blocked_signals;

    function getenv(name: posix_string; undefined: posix_string := "")
		return posix_string renames environment_value_of;
    function in_env(name: posix_string) return boolean
		renames is_environment_variable;

    procedure check_file(file: file_descriptor; name: string) is
	st1: status := get_file_status(file);
	st2: status := get_file_status(to_posix_string(name));
    begin
	if device_id_of(st1) /= device_id_of(st2) or
		file_id_of(st1) /= file_id_of(st2) then
	    fail("file" & file_descriptor'image(file) & " does not refer to "
			& name);
	end if;
    end check_file;

    function is_number(num: string) return boolean is
    begin
	if num'last < num'first then
	    return false;
	end if;
	for p in num'range loop
	    if num(p) not in '0' .. '9' then
		return false;
	    end if;
	end loop;
	return true;
    end is_number;

    not_found: constant integer := integer'last;
    function char_pos(c: character; s: string) return positive is
    begin
	for p in s'range loop
	    if s(p) = c then
		return p;
	    end if;
	end loop;
	return not_found;
    end char_pos;

begin
    if argument_count <= 0 then
	fail("Bad argument count");
    else
	declare
	    c: integer;
	begin
	    c := integer'value(argument(1));
	    if c /= argument_count - 1 then
		fail("Expected argument count wrong");
	    end if;
	exception
	    when constraint_error =>
		fail("Bad count argument: " & argument(1));
	end;
    end if;

    for argnum in 2 .. argument_count loop
	declare
	    arg: string := argument(argnum);
	    space: positive := char_pos(' ', arg);
	    equals: positive := char_pos('=', arg);
	    file: file_descriptor;
	begin
	    if equals /= not_found and then arg(1) in 'A' .. 'Z' then
		if not in_env(to_posix_string(arg(1 .. equals - 1))) then
		    fail(arg(1 .. equals - 1) & " not in the environment");
		elsif getenv(to_posix_string(arg(1 .. equals - 1)))
			/=  to_posix_string(arg(equals + 1 .. arg'last)) then
		    fail("Wrong value for " & arg(1 .. equals - 1) & ": "
				& arg(equals + 1 .. arg'last));
		end if;
	    elsif space /= not_found and then arg(1) in 'A' .. 'Z' and then
			arg(space + 1 .. arg'last) = "undefined" then
		if in_env(to_posix_string(arg(1 .. space - 1))) then
		    fail(arg(1 .. space - 1) & " is not undefined");
		end if;
	    elsif equals /= not_found and then is_number(arg(1 .. equals - 1)) then
		check_file(file_descriptor'value(arg(1 .. equals - 1)),
				arg(equals + 1 .. arg'last));
	    elsif space /= not_found and then is_number(arg(1 .. space - 1))
			and then arg(space + 1 .. arg'last) = "closed" then
		if is_open(file_descriptor'value(arg(1 .. space - 1))) then
		    fail("file descriptor " & arg(1 .. space - 1) & " is open");
		end if;
	    elsif equals /= not_found and then arg(1 .. equals - 1) = "exit"
			and then is_number(arg(equals + 1 .. arg'last)) then
		good_exit_status := exit_status'value(arg(equals + 1 .. arg'last));
	    elsif arg = "blocked SIGNAL_USER_1" then
		if not is_member(blocked, signal_user_1) then
		    fail("signal_user_1 is not blocked");
		end if;
	    elsif arg = "unblocked SIGNAL_USER_1" then
		if is_member(blocked, signal_user_1) then
		    fail("signal_user_1 is blocked");
		end if;
	    else
		fail("Bad argument: " & arg);
	    end if;
	end;
    end loop;

    if error_count > 39 then
	error_count := 39;
    end if;
    if error_count > 0 then
	set_exit_status(exit_status(error_count));
    else
	set_exit_status(good_exit_status);
    end if;
end child;
