--------------------------------------------------------------------------
--  Forest library, version 1.3						--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

-- The POSIX calendar package is a clone of Ada.Calendar which uses the
-- TZ variable to decide which time zone to use.  Under GNAT the Ada.Calendar
-- package already uses the TZ variable, so this package is unnecessary,
-- but is provided for POSIX compatibility.

with Ada.Calendar;
package POSIX_Calendar is

    -- Posix_time is private according to POSIX, but we derive it
    -- from Ada.Calendar.time in the public part of the package to
    -- get the derived operations without any fuss.  The portable way
    -- to convert between posix_time and time is to use the to_time
    -- and to_posix_time functions below.

    type POSIX_Time is new Ada.Calendar.Time;

    function To_Time(Date: POSIX_Time) return Ada.Calendar.Time;

    function To_POSIX_Time(Date: Ada.Calendar.Time) return POSIX_Time;

    subtype Year_Number    is Ada.Calendar.Year_Number;
    subtype Month_Number   is Ada.Calendar.Month_Number;
    subtype Day_Number     is Ada.Calendar.Day_Number;
    subtype Day_Duration   is Ada.Calendar.Day_Duration;

end POSIX_Calendar;
