--------------------------------------------------------------------------
--  Forest library, version 1.3						--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

with POSIX_Private_2; use POSIX_Private_2;
package body POSIX_Permissions is

function umask(Mask: Packed_Permission_Set) return Packed_Permission_Set;
procedure umask(Mask: Packed_Permission_Set);	-- discards result
pragma import(C, umask, "umask");

Access_Bits: constant Packed_Permission_Set := 8#777#;

Umask_Cache: Packed_Permission_Set;


function Get_Allowed_Process_Permissions return Permission_Set is
    Mask:   Packed_Permission_Set;
    Junk:   Packed_Permission_Set;
begin
    -- The allowed process permissions are the complement of the
    -- access bits in umask.  The other bits in both the umask and
    -- the allowed process permissions are normally zero, so we
    -- xor the umask with access_bits to complement the acces bits only.

    -- The use of umask_cache means that normally umask is called only
    -- once.  This is desirable because calling umask twice produces a
    -- race condition in the presence of tasking.

    Mask := umask(Umask_Cache);
    if Mask /= Umask_Cache then
	Umask_Cache := Mask;
	umask(Mask);
    end if;
    return Unpack(Mask xor Access_Bits);
end Get_Allowed_Process_Permissions;

procedure Set_Allowed_Process_Permissions(Permissions : in Permission_Set) is
    Mask:   Packed_Permission_Set;
begin
    Mask := Pack(Permissions) xor Access_Bits;
    Umask_Cache := Mask;
    umask(Mask);
end Set_Allowed_Process_Permissions;

procedure Set_Allowed_Process_Permissions
	(Permissions: Permission_Set;
	 Old_Perms: out Permission_Set)
is
    Mask:   Packed_Permission_Set;
begin
    Mask := Pack(Permissions) xor Access_Bits;
    Umask_Cache := Mask;
    Old_Perms := Unpack(umask(Mask) xor Access_Bits);
end Set_Allowed_Process_Permissions;

begin
    -- initialize umask_cache
    Umask_Cache := umask(0);
    umask(Umask_Cache);
end POSIX_Permissions;
