--------------------------------------------------------------------------
--  Forest library, version 1.3						--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

package body POSIX_Process_Times is

type Struct_Tms_Ptr is access all Struct_Tms;

Start_Time: Tick_Count;		-- Time when this program started.

function C_Times(Buf: Struct_Tms_Ptr) return Tick_Count;
pragma import(C, C_Times, "times");

function Elapsed_Real_Time return Tick_Count is
    Junk: aliased Struct_Tms;
begin
    return C_Times(Junk'unchecked_access) - Start_Time;
end Elapsed_Real_Time;


function Get_Process_Times return Process_Times is
    Result: Process_Times;
begin
    Result.Real_Time := C_Times(Result.Tms'unchecked_access) - Start_Time;
    return Result;
end Get_Process_Times;

function Elapsed_Real_Time_Of(Times: Process_Times) return Tick_Count is
begin
    return Times.Real_Time;
end Elapsed_Real_Time_Of;


function User_CPU_Time_Of(Times: Process_Times) return Tick_Count is
begin
    return Times.Tms.Tms_Utime;
end User_CPU_Time_Of;


function System_CPU_Time_Of(Times: Process_Times) return Tick_Count is
begin
    return Times.Tms.Tms_Stime;
end System_CPU_Time_Of;


function Descendants_User_CPU_Time_Of(Times: Process_Times)
	return Tick_Count is
begin
    return Times.Tms.Tms_Cutime;
end Descendants_User_CPU_Time_Of;


function Descendants_System_CPU_Time_Of(Times: Process_Times)
	return Tick_Count is
begin
    return Times.Tms.Tms_Cstime;
end Descendants_System_CPU_Time_Of;

begin
    -- Set start_time to the current time.  The Elapsed_Real_Time function
    -- returns the current time if called when start_time is zero.
    Start_Time := 0;
    Start_Time := Elapsed_Real_Time;
end POSIX_Process_Times;
