--------------------------------------------------------------------------
--  Forest library, version 1.3						--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

with POSIX_Private_1;
with POSIX_Private_2; use POSIX_Private_2;
with POSIX.POSIX_Private;
with POSIX_Process_Identification.More;
with POSIX_Process_Identification.POSIX_Private;
with POSIX_Process_Environment.POSIX_Private;
with POSIX_Process_Environment.More; use POSIX_Process_Environment.More;
with System;
package body POSIX_Unsafe_Process_Primitives is

use POSIX;
use POSIX.POSIX_Private;
use POSIX_Process_Identification;
use POSIX_Process_Identification.More;
use POSIX_Process_Identification.POSIX_Private;
use POSIX_Process_Environment;
package Env_Private renames POSIX_Process_Environment.POSIX_Private;


function c_fork return Integer;
pragma import(C, c_fork, "fork");


function Fork return Process_ID is
    Pid: Integer;
begin
    Pid := c_fork;
    if Pid = -1 then
	raise POSIX_Error;
    end if;
    return Integer_To_Process_ID(Pid);
end Fork;


procedure c_execve(Filename: System.Address;
		Argv: System.Address; Envp: System.Address);
pragma import(C, c_execve, "execve");


procedure Exec
	(Pathname: POSIX.Pathname;
	 Arg_List: POSIX.POSIX_String_List := POSIX.Empty_String_List) is
begin
    Exec(Pathname, Arg_List, Current_Environment);
end Exec;


procedure Exec
	(Pathname: POSIX.Pathname;
	 Arg_List: POSIX.POSIX_String_List := POSIX.Empty_String_List;
	 Env_List: POSIX_Process_Environment.Environment)
is
    -- The following line should read:
    --     Name: POSIX_String := Pathname & NUL;
    -- but GNAT has trouble mixing this construct with exceptions.
    Name:   POSIX_String(Pathname'first .. Pathname'last + 1);
    use POSIX.POSIX_Private;
    -- The following line should read:
    --     Argv: Argv_Array := Values(Arg_List);
    Argv: Argv_Array (1 .. Length(Arg_List) + 1);
    procedure Set_Argv is
    begin
	Argv := Values(Arg_List);
    end Set_Argv;
begin
    Name(Pathname'first .. Pathname'last) := Pathname;
    Name(Pathname'last + 1) := POSIX_Private_2.NUL;
    Set_Argv;
    c_execve(Name(Name'first)'address, Argv(Argv'first)'address,
		Env_Private.to_C(Env_List));
    raise POSIX_Error;
end Exec;
    

procedure Exec_Search
	(Filename: POSIX.Filename;
	 Arg_List: POSIX.POSIX_String_List := POSIX.Empty_String_List) is
begin
    Exec_Search(Filename, Arg_List, Current_Environment);
end Exec_Search;


procedure Exec_Search
	(Filename: POSIX.Filename;
	 Arg_List: POSIX.POSIX_String_List := POSIX.Empty_String_List;
	 Env_List: POSIX_Process_Environment.Environment)
is
    Contains_Slash: Boolean := false;
begin
    for I in Filename'range loop
	if Filename(I) = '/' then
	    Contains_Slash := true;
	    exit;
	end if;
    end loop;

    if Contains_Slash then
	Exec(Filename, Arg_List, Env_List);
    else
	declare
	    function Make_Path_Name(Directory: POSIX_String;
				    File: POSIX_String)
			return POSIX_String is
	    begin
		if Directory = "" then
		    return File;
		elsif Directory(Directory'last) = '/' then
		    return Directory & File;
		else
		    return Directory & '/' & File;
		end if;
	    end Make_Path_Name;
	    pragma inline(Make_Path_Name);

	    Path: POSIX_String := Environment_Value_Of("PATH", "/bin:/usr/bin");
	    Start: Positive;
	    P: Positive;
	    Err: Error_Code := No_Such_File_Or_Directory;
	begin
	    P := Path'first;
	    loop
		Start := P;
		while P <= Path'last and then Path(P) /= ':' loop
		    P := P + 1;
		end loop;
		declare
		    Pathname: POSIX_String := Make_Path_Name(
					Path(Start .. P - 1), Filename);
		begin
		    Exec(Pathname, Arg_List, Env_List);
		exception
		    when POSIX_Error =>
			null;
		end;
		if Get_Error_Code /= No_Such_File_Or_Directory then
		    Err := Get_Error_Code;
		end if;
		exit when P > Path'last;
		P := P + 1;		-- skip colon
	    end loop;
	end;
    end if;
end Exec_Search;

end POSIX_Unsafe_Process_Primitives;
