--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------

with ada.command_line,
     text_io,
     posix_terminal_functions,
     posix,
     posix_io;
procedure stm is
    use ada.command_line, posix, posix_terminal_functions, text_io;
    tc: terminal_characteristics;
    modes: terminal_modes_set;
begin
    begin
	tc := get_terminal_characteristics(posix_io.standard_error);
    exception
	when posix_error =>
	    put_line("Standard error is not a tty");
	    return;
    end;
    modes := terminal_modes_of(tc);
    if argument_count = 0 then
	for m in terminal_modes loop
	    put(terminal_modes'image(m));
	    put(" => ");
	    put(boolean'image(modes(m)));
	    new_line;
	end loop;
    else -- argument_count /= 0
	for a in 1 .. argument_count loop
	    declare
		arg: string := argument(a);
		m: terminal_modes;
		value: boolean;
	    begin
		case arg(arg'first) is
		    when '+' =>
			value := true;
		    when '-' =>
			value := false;
		    when others =>
			put("Invalid argument: "); put(arg); new_line;
			return;
		end case;
		begin
		    m := terminal_modes'value(arg(2 .. arg'last));
		exception
		    when constraint_error =>
			put("Invalid argument: "); put(arg); new_line;
			return;
		end;
		modes(m) := value;
	    end;
	end loop;
	define_terminal_modes(tc, modes);
	begin
	    set_terminal_characteristics(posix_io.standard_error, tc);
	exception
	    when posix_error =>
		put_line("Could not set terminal modes");
		return;
	end;
    end if;
end stm;
