--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------

-- Test terminal functions
with text_io; use text_io;
with test_pkg; use test_pkg;
with posix; use posix;
with posix_io; use posix_io;
with posix_terminal_functions; use posix_terminal_functions;
with posix_terminal_functions.more; use posix_terminal_functions.more;
procedure terminal is

procedure put is new test_pkg.putint(integer);

procedure tty is
    original_tc: terminal_characteristics;
    tc: terminal_characteristics;
    new_tc: terminal_characteristics;
    modes: terminal_modes_set;
    new_modes: terminal_modes_set;
    test_fd: file_descriptor;
begin
    test("T10");
    original_tc := get_terminal_characteristics(posix_io.standard_error);
    test("T11");
    tc := original_tc;
    modes := terminal_modes_of(tc);
    require(modes = terminal_modes_of(original_tc));
    test("T12");
    require(modes(enable_signals) = true, "enable_signals not set");
    test("T13");
    modes(ignore_break) := true;
    modes(mark_parity_errors) := true;
    modes(perform_output_processing) := true;
    modes(echo_kill) := true;
    modes(no_flush) := true;
    define_terminal_modes(tc, modes);
    test("T14");
    set_terminal_characteristics(posix_io.standard_error, tc, after_output);
    test("T15");
    new_tc := get_terminal_characteristics(posix_io.standard_error);
    new_modes := terminal_modes_of(new_tc);
    require(new_modes = modes);
    test("T16");
    modes(ignore_break) := false;
    modes(mark_parity_errors) := false;
    modes(perform_output_processing) := false;
    modes(echo_kill) := false;
    modes(no_flush) := false;
    define_terminal_modes(new_tc, modes);
    set_terminal_characteristics(posix_io.standard_error, new_tc);
    test("T17");
    tc := new_tc;
    new_tc := get_terminal_characteristics(posix_io.standard_error);
    new_modes := terminal_modes_of(new_tc);
    require(new_modes = modes);
    test("T18");
    define_minimum_input_count(tc, 17);
    require(minimum_input_count_of(tc) = 17, "input count wrong");
    define_input_time(tc, 0.5);
    put("input time ="); put(duration'image(input_time_of(tc))); nl;
    require(input_time_of(tc) = 0.5, "input time wrong");
    set_terminal_characteristics(posix_io.standard_error, tc,
				 after_output_and_input);
    test("T19");
    new_tc := get_terminal_characteristics(posix_io.standard_error);
    new_modes := terminal_modes_of(new_tc);
    require(new_modes = modes, "modes incorrect");
    put("count = "); put(minimum_input_count_of(new_tc));
    nl;
    require(minimum_input_count_of(new_tc) = minimum_input_count_of(tc), "count wrong");
    require(input_time_of(new_tc) = input_time_of(tc), "time wrong");
    test("T20");
    declare
	uninitialized: terminal_characteristics;
    begin
	set_terminal_characteristics(posix_io.standard_error, uninitialized);
	except;
    exception
	when posix_error =>
	    check_error(invalid_argument);
    end;
    test("T21");
    begin
	define_minimum_input_count(tc, 30000);
	except;
    exception
	when posix_error =>
	    check_error(invalid_argument);
    end;
    require(minimum_input_count_of(tc) = 17);
    test("T22");
    begin
	define_input_time(tc, 100.0);
	except;
    exception
	when posix_error =>
	    check_error(invalid_argument);
    end;
    require(input_time_of(tc) = 0.5);
    test("T23");
    test_fd := open("test_file", read_write);
    begin
	set_terminal_characteristics(test_fd, tc);
	except;
    exception
	when posix_error =>
	    check_error(inappropriate_io_control_operation);
    end;
    test("T24");
    begin
	tc := get_terminal_characteristics(test_fd);
	except;
    exception
	when posix_error =>
	    check_error(inappropriate_io_control_operation);
    end;
    close(test_fd);
    test("T30");
    set_terminal_characteristics(posix_io.standard_error, original_tc);
exception
    when posix_error =>
	fail("Unexpected POSIX_Error: " & image(get_error_code));
end tty;

begin -- terminal
    put("Test program started."); nl;

    -- run tests --
    tty;
    done;
end terminal;
