--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------

with text_io; use text_io;
package body test_pkg is

use posix;


procedure test(label: string) is
begin
    put("Test "); put(label); nl;
end test;


procedure fail is
begin
    put("Test failed."); nl;
    error_count := error_count + 1;
end fail;

procedure fail(message: string) is
begin
    put("Test failed: "); put(message); nl;
    error_count := error_count + 1;
end fail;


procedure require(condition: boolean) is
begin
    if not condition then
	fail("require");
    end if;
end require;


procedure require(condition: boolean; message: string) is
begin
    if not condition then
	fail(message);
    end if;
end require;


procedure except is
begin
    fail("Exception expected");
end except;


procedure unexpected is
begin
    fail("Unexpected POSIX_Error: " & image(get_error_code));
end unexpected;


procedure check_error(expect: posix.error_code) is
    use posix;
    e: error_code := get_error_code;
begin
    if e /= expect then
	fail("Error code is " & image(e));
	put_line("      Expected error code: " & image(expect));
	flush;
    end if;
end check_error;


procedure done is
begin
    new_line;
    if error_count = 0 then
	put("Tests completed successfully.");
    else
	put("Tests finished:"); put(natural'image(error_count));
	put(" error");
	if error_count /= 1 then put('s'); end if;
    end if;
    new_line;
    flush;
end done;


-- generic type inttype is range <>;
procedure putint(item: inttype) is
    im: constant string := inttype'image(item);
    first: positive;
begin
    first := 1;
    if im(1) = ' ' then
	first := 2;
    end if;
    put(im(first .. im'last));
end putint;


procedure nl is
begin
    new_line;
    flush;
end nl;

end test_pkg;
