--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------

with posix;
package test_pkg is

    -- This package contains utility routines which are useful for
    -- running tests.

    error_count: natural := 0;		-- number of errors so far.


    -- Print a message indicating that a test is about to be performed.
    -- The label should be unique so that if the test fails it is
    -- possible to find the code for the test by grepping through
    -- the source.
    procedure test(label: string);

    -- Record the fact that the test failed.
    procedure fail;

    -- As above, but with an error message.
    procedure fail(message: string);

    -- The test fails if the argument is false.
    procedure require(condition: boolean);

    -- The test fails if the argument is false.
    procedure require(condition: boolean; message: string);

    -- The following call should be placed after a call which is supposed
    -- to exit with an exception.  In other words, it should be called
    -- when an exception is expected but did not occur.
    procedure except;

    -- The following should appear in an exception handler for POSIX_Error
    -- when an exception is not expected.
    procedure unexpected;

    -- Validate an error code.
    procedure check_error(expect: posix.error_code);

    -- Called when the testing is complete.
    procedure done;

    -- Output an value of inttype, with no leading spaces.
    generic
	type inttype is range <>;
    procedure putint(item: inttype);

    -- Output a new_line and flush the output buffer.
    procedure nl;

end test_pkg;
