/************************************************************************
 *  Forest library, version 1.4beta					*
 *  Copyright (C) 1995 by Kenneth Almquist				*
 *									*
 *  You may redistribute or modify this software under the terms	*
 *  given in the License for the Forest Library, which appears in	*
 *  the file named LICENSE.						*
 ************************************************************************/

/*
 * This program writes various constants from C header files to the
 * file cdefs.m4.
 */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <unistd.h>
#include <limits.h>
#include <signal.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <termios.h>
#include <time.h>


/* CHECK THIS */
#ifdef __linux
#define _PC_LINK_MAX		_PC_LINK_MAX
#define _PC_MAX_CANON		_PC_MAX_CANON
#define _PC_MAX_INPUT		_PC_MAX_INPUT
#define _PC_NAME_MAX		_PC_NAME_MAX
#define _PC_PATH_MAX		_PC_PATH_MAX
#define _PC_PIPE_BUF		_PC_PIPE_BUF
#define _PC_CHOWN_RESTRICTED	_PC_CHOWN_RESTRICTED
#define _PC_NO_TRUNC		_PC_NO_TRUNC
#define _PC_VDISABLE		_PC_VDISABLE
#define _SC_ARG_MAX		_SC_ARG_MAX
#define _SC_CHILD_MAX		_SC_CHILD_MAX
#define _SC_CLK_TCK		_SC_CLK_TCK
#define _SC_NGROUPS_MAX		_SC_NGROUPS_MAX
#define _SC_OPEN_MAX		_SC_OPEN_MAX
#define _SC_STREAM_MAX		_SC_STREAM_MAX
#define _SC_TZNAME_MAX		_SC_TZNAME_MAX
#define _SC_JOB_CONTROL		_SC_JOB_CONTROL
#define _SC_SAVED_IDS		_SC_SAVED_IDS
#define _SC_VERSION		_SC_VERSION
#define _SC_PAGESIZE		_SC_PAGESIZE
#define _SC_BC_BASE_MAX		_SC_BC_BASE_MAX
#define _SC_BC_DIM_MAX		_SC_BC_DIM_MAX
#define _SC_BC_SCALE_MAX	_SC_BC_SCALE_MAX
#define _SC_BC_STRING_MAX	_SC_BC_STRING_MAX
#define _SC_COLL_WEIGHTS_MAX	_SC_COLL_WEIGHTS_MAX
#define _SC_EQUIV_CLASS_MAX	_SC_EQUIV_CLASS_MAX
#define _SC_EXPR_NEST_MAX	_SC_EXPR_NEST_MAX
#define _SC_LINE_MAX		_SC_LINE_MAX
#define _SC_RE_DUP_MAX		_SC_RE_DUP_MAX
#define _SC_2_VERSION		_SC_2_VERSION
#define _SC_2_C_BIND		_SC_2_C_BIND
#define _SC_2_C_DEV		_SC_2_C_DEV
#define _SC_2_FORT_DEV		_SC_2_FORT_DEV
#define _SC_2_FORT_RUN		_SC_2_FORT_RUN
#define _SC_2_SW_DEV		_SC_2_SW_DEV
#define _SC_2_LOCALEDEF		_SC_2_LOCALEDEF
#endif


struct lval {
    const char *name;
    long value;
    int mode;
};

#include "cconst.h"	/* defines longconst array */

FILE *fp;


main() {
    const struct lval *lp;
    char val[32];
    int i;
    int errflag;

    /* Open the output file. */
    if ((fp = fopen("cdefs.m4", "w")) == NULL) {
	fputs("Cannot create cdefs.m4\n", stderr);
	exit(2);
    }

    /* Write values of constants. */
    errflag = 0;
    for (lp = longconst ; lp->name ; lp++) {
	switch (lp->mode) {
	    case 0:
		sprintf(val, "[%ld]", lp->value);
		break;
	    case 1:
		sprintf(val, "[[8#%o#]]", lp->value);
		break;
	    case 2:
		for (i = 0 ; ; i++) {
		    if ((1 << i) == 0) {
			fprintf(stderr, "Bad definition for %s: 0x%lX\n",
				lp->name, lp->value);
			errflag++;
			i = 0;
			break;
		    }
		    if (lp->value == (1 << i)) {
			break;
		    }
		}
		sprintf(val, "[Element_%d]", i);
		break;
	    case 3:
		if (lp->value)
		    strcpy(val, "[true]");
		else
		    strcpy(val, "[false]");
		break;
	    case 4:
		fprintf(stderr, "%s undefined\n", lp->name);
		errflag++;
		strcpy(val, "[0]");
		break;
	}
	fprintf(fp, "define(%s, %s)\n", lp->name, val);
    }
    /*
     * Ticks_per_second is a compile time parameter in POSIX.5 but a
     * run time parameter in POSIX.1.  (Don't ask me why.)  So it
     * requires special treatment.
     */
#ifdef _SC_CLK_TCK
#define TICKS_PER_SECOND sysconf(_SC_CLK_TCK)
#else
#ifdef CLK_TCK
#define TICKS_PER_SECOND CLK_TCK
#else
#ifdef HZ
#define TICKS_PER_SECOND HZ
#else
    fprintf(stderr, "CLK_TCK undefined\n");
    errflag++;
#define TICKS_PER_SECOND 100
#endif
#endif
#endif
    fprintf(fp, "define(CLK_TCK, %ld)\n", (long)(TICKS_PER_SECOND));

    exit(errflag? 2 : 0);
}
