--------------------------------------------------------------------------
--  Forest library, version 1.4beta					--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

with Interfaces.C.Strings;
with POSIX_Private_2; use POSIX_Private_2;
package body POSIX.More is

use Ada.Exceptions;

function strerror(Error: Error_Code) return Interfaces.C.Strings.Chars_Ptr;
pragma import(C, strerror);

generic
    type Return_Type is range <>;
    Function_Name: String;
    Skip_To_Count: Boolean;
function Get_Error_Data(X: Exception_Occurrence) return Return_Type;
    -- Implements both Error_Code_Of and Transfer_Count_Of.

function Get_Error_Data(X: Exception_Occurrence) return Return_Type is
begin
    if Exception_Identity(X) /= POSIX_Error'identity then
	Raise_POSIX_Error(Function_Name, Invalid_Argument, 0);
    end if;
    declare
	Msg: constant String := Exception_Message(X);
	P: Integer;
	Value: Return_Type := 0;
	Minus: Boolean := false;
	Error_Len: constant Integer := 6;    -- length of "error ";
    begin
	P := Msg'last + 1 - Error_Len;
	loop
	    if P <= Msg'first then
		return 0;
	    end if;
	    exit when Msg(P .. P + Error_Len - 1) = "error ";
	    P := P - 1;
	end loop;
	P := P + Error_Len;
	if Skip_To_Count then	-- Skip to the transfer count.
	    while P <= Msg'last and then Msg(P) /= '(' loop
		P := P + 1;
	    end loop;
	    P := P + 1;
	end if;
	if P <= Msg'last and then Msg(P) = '-' then
	    Minus := true;
	    P := P + 1;
	end if;

	-- On two's complement machines, there is one extra negative value.
	-- To avoid overflow, we set Value to the negation of the value
	-- represented by the digits.  We then negate Value if the digits
	-- were not preceded by a minus sign.
	while P <= Msg'last and then (Msg(P) >= '0' and Msg(P) <= '9') loop
	    Value := Value * 10 - (Character'pos(Msg(P)) - Character'pos('0'));
	    P := P + 1;
	end loop;
	if not Minus then
	    Value := - Value;
	end if;
	return Value;
    end;
end Get_Error_Data;


function Error_Code_Of(X: Exception_Occurrence) return Error_Code is
    function Do_Work is new
	Get_Error_Data(Error_Code, "Error_Code_Of", false);
begin
    -- A bug in GNAT 3.01 (tracking number 2424-01) can cause the exception
    -- message to be set to the null string when an exception is reraised.
    -- To recover from this, we try the following:
    if Exception_Identity(X) = POSIX_Error'identity
      and Exception_Message(X) = "" then
	return Get_Error_Code;
    end if;

    return Do_Work(X);
end Error_Code_Of;


function Transfer_Count_Of(X: Exception_Occurrence) return IO_Count is
    function Do_Work is new
	Get_Error_Data(IO_Count, "Transfer_Count_Of", true);
begin
    return Do_Work(X);
end Transfer_Count_Of;


function Error_String(Error: Error_Code) return String is
begin
    return Interfaces.C.Strings.Value(strerror(Error));
end Error_String;

end POSIX.More;
