--------------------------------------------------------------------------
--  Forest library, version 1.4beta					--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

package body POSIX_Calendar is

use Ada.Calendar;
use Ada.Calendar.POSIX_Private;

function To_Time(Date: POSIX_Time) return Ada.Calendar.Time is
begin
    return To_Time(Time_T(Date));
end To_Time;

function To_POSIX_Time(Date: Ada.Calendar.Time) return POSIX_Time is
begin
    return POSIX_Time(Ada.Calendar.POSIX_Private.To_POSIX_Time(Date));
end To_POSIX_Time;

function Clock return POSIX_Time is
begin
    return To_POSIX_Time(Ada.Calendar.Clock);
end Clock;


function Year(Date:  POSIX_Time) return Year_Number is
begin
    return Ada.Calendar.Year(To_Time(Date));
end Year;

function Month(Date:  POSIX_Time) return Month_Number is
begin
    return Ada.Calendar.Month(To_Time(Date));
end Month;

function Day(Date:  POSIX_Time) return Day_Number is
begin
    return Ada.Calendar.Day(To_Time(Date));
end Day;

function Seconds(Date:  POSIX_Time) return Day_Duration is
begin
    return Ada.Calendar.Seconds(To_Time(Date));
end Seconds;


procedure Split
    (Date:     POSIX_Time;
     Year:     out Year_Number;
     Month:    out Month_Number;
     Day:      out Day_Number;
     Seconds:  out Day_Duration) is
begin
    Ada.Calendar.Split(To_Time(Date), Year, Month, Day, Seconds);
end Split;

function Time_Of
    (Year:     Year_Number;
     Month:    Month_Number;
     Day:      Day_Number;
     Seconds:  Day_Duration := 0.0)
	    return POSIX_Time is
begin
    return To_POSIX_Time(Ada.Calendar.Time_Of(Year, Month, Day, Seconds));
end  Time_Of;


function "+" (Left: POSIX_Time; Right: Duration) return POSIX_Time is
begin
    return POSIX_Time(Duration(Left) + Right);
exception
    when Constraint_Error =>
	raise Time_Error;
end "+";

function "+" (Left: Duration; Right: POSIX_Time) return POSIX_Time is
begin
    return POSIX_Time(Left + Duration(Right));
exception
    when Constraint_Error =>
	raise Time_Error;
end "+";

function "-" (Left: POSIX_Time; Right: Duration) return POSIX_Time is
begin
    return POSIX_Time(Duration(Left) - Right);
exception
    when Constraint_Error =>
	raise Time_Error;
end "-";

function "-" (Left: POSIX_Time; Right: POSIX_Time) return Duration is
begin
    return Duration(Left) - Duration(Right);
end "-";


function "<" (Left, Right: POSIX_Time) return Boolean is
begin
    return Time_T(Left) < Time_T(Right);
end "<";

function "<=" (Left, Right: POSIX_Time) return Boolean is
begin
    return Time_T(Left) <= Time_T(Right);
end "<=";

function ">" (Left, Right: POSIX_Time) return Boolean is
begin
    return Time_T(Left) > Time_T(Right);
end ">";

function ">=" (Left, Right: POSIX_Time) return Boolean is
begin
    return Time_T(Left) >= Time_T(Right);
end ">=";

end POSIX_Calendar;
