--------------------------------------------------------------------------
--  Forest library, version 1.4beta					--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

-- The POSIX calendar package is a clone of Ada.Calendar which uses the
-- TZ variable to decide which time zone to use.  Under GNAT the Ada.Calendar
-- package already uses the TZ variable, so this package is unnecessary,
-- but is provided for POSIX compatibility.

with Ada.Calendar.POSIX_Private;
package POSIX_Calendar is

    type POSIX_Time is private;

    function To_Time(Date: POSIX_Time) return Ada.Calendar.Time;
	-- Exceptions:  none
	-- Convert a POSIX_Time to an Ada Time.

    function To_POSIX_Time(Date: Ada.Calendar.Time) return POSIX_Time;
	-- Exceptions:  Contraint_Error
	-- Convert an Ada Time to a POSIX_Time.

    function Clock return POSIX_Time;
	-- Returns the current time.

    subtype Year_Number    is Ada.Calendar.Year_Number;
    subtype Month_Number   is Ada.Calendar.Month_Number;
    subtype Day_Number     is Ada.Calendar.Day_Number;
    subtype Day_Duration   is Ada.Calendar.Day_Duration;


    function Year    (Date:  POSIX_Time) return Year_Number;
    function Month   (Date:  POSIX_Time) return Month_Number;
    function Day     (Date:  POSIX_Time) return Day_Number;
    function Seconds (Date:  POSIX_Time) return Day_Duration;

    procedure Split
	(Date:	   POSIX_Time;
	 Year:	   out Year_Number;
	 Month:	   out Month_Number;
	 Day:	   out Day_Number;
	 Seconds:  out Day_Duration);

    function Time_Of
	(Year:	   Year_Number;
	 Month:	   Month_Number;
	 Day:	   Day_Number;
	 Seconds:  Day_Duration := 0.0)
		return POSIX_Time;

    function "+" (Left:  POSIX_Time;  Right:  Duration)   return POSIX_Time;
    function "+" (Left:  Duration;    Right:  POSIX_Time) return POSIX_Time;
    function "-" (Left:  POSIX_Time;  Right:  Duration)   return POSIX_Time;
    function "-" (Left:  POSIX_Time;  Right:  POSIX_Time) return Duration;
    pragma inline ("+");
    pragma inline ("-");

    function "<"  (Left, Right:  POSIX_Time) return Boolean;
    function "<=" (Left, Right:  POSIX_Time) return Boolean;
    function ">"  (Left, Right:  POSIX_Time) return Boolean;
    function ">=" (Left, Right:  POSIX_Time) return Boolean;
    pragma inline ("<");
    pragma inline ("<=");
    pragma inline (">");
    pragma inline (">=");

    Time_Error: exception renames Ada.Calendar.Time_Error;

private
    type POSIX_Time is new Ada.Calendar.POSIX_Private.Time_T;

end POSIX_Calendar;
