--------------------------------------------------------------------------
--  Forest library, version 1.4beta					--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

with POSIX,
     POSIX_Process_Identification;
package POSIX_Group_Database is

    type Group_Database_Item is private;

    type Group_ID_List is private;

    -- operations to get a Group_Database_Item
    -- At one point POSIX.5 uses the name Group_Database_Item_Of for
    -- these, but the name Get_Group_Database_Item appears to be the
    -- intended name.

    function Get_Group_Database_Item
            (GID: POSIX_Process_Identification.Group_ID)
        return Group_Database_Item;
	-- Exceptions:  posix_error.
	-- Returns information about a group from the group data base.

    function Get_Group_Database_Item(Name: POSIX.POSIX_String)
        return Group_Database_Item;
	-- Exceptions:  posix_error.
	-- Returns information about a group from the group data base.

    -- operations to get information from a Group_Database_Item

    function Group_Name_Of(DB_Item: Group_Database_Item)
        return POSIX.POSIX_String;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Returns the name of the group.

    function Group_ID_Of(DB_Item: Group_Database_Item)
        return POSIX_Process_Identification.Group_ID;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Returns the group ID of the group.

    function Group_ID_List_Of(DB_Item: Group_Database_Item)
        return Group_ID_List;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Returns the list of users in the group.  (This is the list
	-- which appears in the /etc/group file or equivalent.  It
	-- does not necessarily include every user who is placed in
	-- the group when s/he logs in.)


    -- iterator over the Group_ID_List

    generic
        with procedure Action
                     (ID:    POSIX.POSIX_String;
		      Quit:  in out Boolean);
    procedure For_Every_Member(List: Group_ID_List);
	-- exceptions:  none except for those raised by Action.
	-- Calls Action for each user name in the list.

    function Length(Member_List: Group_ID_List)
        return Natural;
	-- exceptions:  none
	-- Returns the number of user names in the list.


private
    type Item_Count is range 0 .. 30;
    subtype Item_Index is Item_Count range 1 .. Item_Count'last;
    subtype User_Name_Length is Natural range 0 .. 127;
    type User_Length_Array is array (Item_Index) of User_Name_Length;
    -- The following pragma doesn't do anything under GNAT:
    -- pragma pack(user_length_array);
    type Member_List is record
	Num_Items:    Item_Count;
	User_Length:  User_Length_Array;
	Space:	      POSIX.POSIX_String(1 .. 240);
    end record;

    subtype Name_Length is Natural range 0 .. 16;
    type Group_Database_Item is record
	Valid:	   Boolean := false;
	Name_Len:  Name_Length;
	Name:	   POSIX.POSIX_String(1 .. Name_Length'last);
	ID:	   POSIX_Process_Identification.Group_ID;
	Users:	   Member_List;
    end record;

    type Group_ID_List is record
	Valid:	Boolean := false;
	Users:	Member_List;
    end record;

end POSIX_Group_Database;
