--------------------------------------------------------------------------
--  Forest library, version 1.4beta					--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

with POSIX;
package POSIX_Permissions is

    -- NOTICE:  S_ISVTX is not included in the POSIX spec, and
    -- should not be used by (normal) clients of this package.
    type Permission is (Others_Execute, Others_Write, Others_Read,
			Group_Execute,  Group_Write,  Group_Read,
			Owner_Execute,  Owner_Write,  Owner_Read,
			posix_private_S_ISVTX, Set_Group_ID, Set_User_ID);

    type Permission_Set is array (Permission) of Boolean;
    -- Packing the array causes problems in GNAT 2.06 and GNAT 2.07.
    --pragma pack(Permission_Set);

    Owner_Permission_Set: constant Permission_Set := (
          Owner_Read | Owner_Write | Owner_Execute => true,
          others => false);

    Group_Permission_Set: constant Permission_Set := (
          Group_Read | Group_Write | Group_Execute => true,
          others => false);

    Others_Permission_Set: constant Permission_Set := (
	  Others_Read | Others_Write | Others_Execute => true,
          others => false);

    Access_Permission_Set: constant Permission_Set := (
          Owner_Read  | Owner_Write  | Owner_Execute => true,
          Group_Read  | Group_Write  | Group_Execute => true,
	  Others_Read | Others_Write | Others_Execute => true,
          others => false);

    Set_Group_ID_Set: constant Permission_Set := (
          Set_Group_ID => true,
          others => false);

    Set_User_ID_Set: constant Permission_Set := (
          Set_User_ID => true,
          others => false);

    -- POSIX Permission-oriented operations

    function Get_Allowed_Process_Permissions return Permission_Set;
	-- Exceptions:  none.
	-- Return the allowed process permissions for this process.  The
	-- allowed process permissions can be thought of as the complement
	-- of the bits returned by the C language function umask.  When
	-- a file is created, a permission is set to true only if it
	-- appears in the allowed process permissions and it is requested
	-- in the operation that creates the file.

    procedure Set_Allowed_Process_Permissions(Permissions: Permission_Set);
	-- Exceptions:  none.
	-- Set the allowed process permissions.

    procedure Set_Allowed_Process_Permissions
	    (Permissions: Permission_Set;
	     Old_Perms: out Permission_Set);
	-- Exceptions:  none.
	-- Set the allowed process permissions, and save the old value in
	-- old_perms.

end POSIX_Permissions;
