--------------------------------------------------------------------------
--  Forest library, version 1.4beta					--
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  You may redistribute or modify this software under the terms	--
--  given in the License for the Forest Library, which appears in	--
--  the file named LICENSE.						--
--------------------------------------------------------------------------

with POSIX,
     POSIX_Process_Identification;
package POSIX_User_Database is

    type User_Database_Item is private;

    -- operations to Get User_Database_Item

    function Get_User_Database_Item(UID: POSIX_Process_Identification.User_ID)
        return User_Database_Item;
	-- exceptions:  posix_error.
	-- Locate a user data base item (traditionally known as a password
	-- file entry) using the user id.

    function Get_User_Database_Item(Name: POSIX.POSIX_String)
        return User_Database_Item;
	-- exceptions:  posix_error.
	-- Locate a user data base item using the user name.

    -- operations to get information From a User_Database_Entry

    function User_Name_Of(DB_Item: User_Database_Item)
        return POSIX.POSIX_String;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Return the user name.

    function User_ID_Of(DB_Item: User_Database_Item)
        return POSIX_Process_Identification.User_ID;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Return the user ID.

    function Group_ID_Of(DB_Item: User_Database_Item)
        return POSIX_Process_Identification.Group_ID;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Return the group that the user is in when s/he logs in.

    function Initial_Directory_Of(DB_Item: User_Database_Item)
        return POSIX.POSIX_String;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Return the user's home directory.

    function Initial_Program_Of(DB_Item: User_Database_Item)
        return POSIX.POSIX_String;
	-- exceptions:  posix_error (when DB_Item not initialized).
	-- Return the user's shell.

private
    type User_Database_Item is record
	Valid:		   Boolean := false;
	User_ID:	   POSIX_Process_Identification.User_ID;
	Group_ID:	   POSIX_Process_Identification.Group_ID;
	User_Name_Length:  Natural;
	User_Name:	   POSIX.POSIX_String(1 .. 16);
	Directory_Length:  Natural;
	Directory:	   POSIX.POSIX_String(1 .. 80);
	Program_Length:	   Natural;
	Program:	   POSIX.POSIX_String(1 .. 96);
    end record;
end POSIX_User_Database;
