--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------


-- (progn (defun insert-fail () (interactive) (beginning-of-line) (insert "    begin\n\t\n\texcept;\n    exception\n\twhen posix_error =>\n\t    check_error();\n    end;\n") (backward-char 74)) (quote
--) (local-set-key "\^cf" 'insert-fail))
-- (progn (save-excursion (replace-regexp "^t  *\\([^ \n]*\\)" "    test(\"\\1\");")) (save-excursion (replace-regexp "\\<r(" "require(")))
with test_pkg; use test_pkg;
with text_io; use text_io;
with posix; use posix;
with posix.more; use posix.more;
with posix_permissions; use posix_permissions;
with posix_files; use posix_files;
with posix_files.more; use posix_files.more;
with posix_file_status; use posix_file_status;
with posix_file_status.more; use posix_file_status.more;
with posix_io; use posix_io;
with posix_process_identification; use posix_process_identification;
with posix_process_identification.more; use posix_process_identification.more;
with posix_process_environment; use posix_process_environment;
with posix_process_environment.more; use posix_process_environment.more;
with posix_process_primitives; use posix_process_primitives;
with posix_unsafe_process_primitives; use posix_unsafe_process_primitives;
with posix_file_locking;
with forest_file_locking;
with posix_signals; use posix_signals;
with posix_calendar; use posix_calendar;
with posix_user_database; use posix_user_database;
with posix_group_database; use posix_group_database;
with posix_configurable_system_limits; use posix_configurable_system_limits;
with ada.calendar; use ada.calendar;
with ada.io_exceptions;
with ada.exceptions;
with file_io, signals;
procedure main is

package constants is

-- process is run with "umask 022"
initial_umask: constant permission_set :=
	    (owner_read | owner_write | owner_execute
		    | group_read | group_execute
		    | others_read | others_execute => true,
	     others => false);

end constants;

use constants;


procedure put is new test_pkg.putint(integer);
procedure put is new test_pkg.putint(exit_status);
procedure put is new test_pkg.putint(io_offset);
procedure put is new test_pkg.putint(io_count);
procedure put is new test_pkg.putint(forest_file_locking.lock_length);


-- Output a time value.  (Why isn't this build into Ada?)
procedure put_time(t: ada.calendar.time) is
    year: ada.calendar.year_number;
    month: ada.calendar.month_number;
    day: ada.calendar.day_number;
    time_of_day: duration;
    tod: long_float;
    hour: integer;
    minute: integer;
    second: integer;
    usec: integer;

    function floor(x: long_float) return long_float renames long_float'floor;

    procedure putpad(i: integer; width: positive) is
	im: constant string := integer'image(i);
	first: positive := 1;
	len: integer;
    begin
	if im(first) = ' ' then
	    first := first + 1;
	end if;
	len := im'last - first + 1;
	while len < width loop
	    put('0');
	    len := len + 1;
	end loop;
	put(im(first .. im'last));
    end putpad;
begin -- put_time
    split(t, year, month, day, time_of_day);
    tod := long_float(time_of_day);
    hour := integer(floor(tod / 3600.0));
    tod := tod - (long_float(hour) * 3600.0);
    minute := integer(floor(tod / 60.0));
    tod := tod - (long_float(minute) * 60.0);
    second := integer(floor(tod));
    tod := tod - floor(tod);
    usec := integer(floor(tod * 1_000_000.0));
    put(month); put("/"); putpad(day, 2); put("/"); putpad(year - 1900, 2);
    put(" "); putpad(hour, 2); put(":"); putpad(minute, 2);
    put(":"); putpad(second, 2); put("."); putpad(usec, 6);
end put_time;


procedure put_posix_time(t: posix_time) is
    year: posix_calendar.year_number;
    month: posix_calendar.month_number;
    day: posix_calendar.day_number;
    time_of_day: duration;
    tod: long_float;
    hour: integer;
    minute: integer;
    second: integer;
    usec: integer;

    function floor(x: long_float) return long_float renames long_float'floor;

    procedure putpad(i: integer; width: positive) is
	im: constant string := integer'image(i);
	first: positive := 1;
	len: integer;
    begin
	if im(first) = ' ' then
	    first := first + 1;
	end if;
	len := im'last - first + 1;
	while len < width loop
	    put('0');
	    len := len + 1;
	end loop;
	put(im(first .. im'last));
    end putpad;
begin -- put_posix_time
    split(t, year, month, day, time_of_day);
    tod := long_float(time_of_day);
    hour := integer(floor(tod / 3600.0));
    tod := tod - (long_float(hour) * 3600.0);
    minute := integer(floor(tod / 60.0));
    tod := tod - (long_float(minute) * 60.0);
    second := integer(floor(tod));
    tod := tod - floor(tod);
    usec := integer(floor(tod * 1_000_000.0));
    put(month); put("/"); putpad(day, 2); put("/"); putpad(year - 1900, 2);
    put(" "); putpad(hour, 2); put(":"); putpad(minute, 2);
    put(":"); putpad(second, 2); put("."); putpad(usec, 6);
end put_posix_time;


-- Output a permission set.
procedure put_perm(p: permission_set) is
begin
    for i in reverse permission loop
	if p(i) then
	    put('T');
	else
	    put('F');
	end if;
    end loop;
end put_perm;



procedure basic is	-- stuff we want to test first
    is_super_user: exception;
    c: character;

    procedure failed_operation(arg1: posix_string; arg2: posix_string) is
	s: constant posix_string := arg1 & arg2;
    begin
	if s = "raise exception" then
	    set_error_code(too_many_links);
	    put_line("raising posix_error");
	    raise posix_error;
	else
	    put("s = "); put(to_string(s)); nl;
	    fail;
	end if;
    end failed_operation;

    procedure real_failure is
	uninitialized:  status;
	junk:           boolean;
    begin
	junk := is_directory(uninitialized);	-- Should raise POSIX_Error.
    end real_failure;

begin
    test("B10");
    if get_real_user_id = super_user_id then
	fail("get_real_user_id returned super_user_id");
	put("For security reasons, the test program should not be run as root");
	nl;
	raise is_super_user;
    end if;
    test("B11");	-- test To_String
    declare
	ps: posix_string := "to_string test data";
	s: string := to_string(ps);
    begin
	require(s'first = 1, "bad s'first");
	require(s = "to_string test data", "bad value");
    end;
    test("B12");	-- test To_POSIX_String
    declare
	s: string := "to_posix_string test data";
	ps: posix_string := to_posix_string(s);
    begin
	require(ps'first = 1, "bad ps'first");
	require(ps = "to_posix_string test data", "bad value");
    end;
    -- The following test is commented out because it produces a core
    -- dump under SunOS 4.1.1 with GNAT 2.07.
--    test("B13");	-- test exception handling
--    begin
--	failed_operation("raise ", "exception");
--	except;
--    exception
--	when posix_error =>
--	    put_line("got posix_error as expected");
--	    check_error(too_many_links);
--    end;
    test("B14");
    begin
	real_failure;
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
end basic;


-- Test handling of POSIX_String_List.
procedure string_list is
    l: posix_string_list;
    third: posix_string (12 .. 16) := "third";
begin
    test("B20");
    require(length(l) = 0);
    test("B21");
    posix.append(l, "first");
    posix.append(l, "second");
    posix.append(l, third);
    test("B22");
    require(length(l) = 3);
    test("B23");
    declare
	pos: positive := 1;
	procedure test_val(item: posix_string; quit: in out boolean) is
	    ok: boolean;
	begin
	    case pos is
		when 1 => ok := item = "first" and item'first = 1;
		when 2 => ok := item = "second" and item'first = 1;
		when 3 => ok := item = "third" and item'first = 12;
		when others => ok := false;
	    end case;
	    if not ok then
		put("item number "); put(pos); nl;
		put("item'first="); put(item'first); nl;
		put("item="); put(to_string(item)); nl;
		fail("invalid list item");
	    end if;
	    pos := pos + 1;
	end test_val;
	procedure test_list is new for_every_item(test_val);
    begin
	test_list(l);
	require(pos = 1 + 3, "Item count");
    end;
    test("B24");
    require(value(l, 1) = "first", "first item incorrect");
    require(value(l, 2) = "second", "second item incorrect");
    require(value(l, 3) = "third", "third item incorrect");
--  put("value(l, 1) = """); put(to_string(value(l, 1))); put(""""); nl;
--  put("value(l, 2) = """); put(to_string(value(l, 2))); put(""""); nl;
--  put("value(l, 3) = """); put(to_string(value(l, 3))); put(""""); nl;
    test("B25");
    begin
	put(to_string(value(l, 4)));	-- 4 is out of range
	except;
    exception
	when constraint_error =>
	    null;
    end;
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end string_list;


-- Test handling of environments.  (This does not include passing them
-- to child processes; that is tested as part of the testing of the
-- process primitives.)

procedure environ is
    type posix_string_ptr is access all posix_string;
    type env_val is record
	name: posix_string_ptr;
	value: posix_string_ptr;
    end record;
    type env_vals is array (positive range <>) of env_val;
    -- try a string whose lower bound is not one.
    SKXSJL: aliased posix_string (10 .. 15) := "SKXSJL";
    -- Note: the following hack is necessary because gnat 2.00 doesn't
    -- like new'("").  We should verify that the problem is gone in 2.02.
    empty_string: aliased posix_string := "";
    data: constant env_vals := (
	(new posix_string'("XXX"), new posix_string'("value of XXX")),
	(new posix_string'("ABCDEFG"), new posix_string'("value of ABCDEFG")),
	(new posix_string'("EMPTY_VALUE"), empty_string'access),
	(new posix_string'("SJKRLS"), new posix_string'("value of SJKRLS")),
	(SKXSJL'access, SKXSJL'access),
	(new posix_string'("LONG_NAME_WITH_A_VERY_LONG_VALUE"),
		new posix_string'("This is the value associated with the very long name.  I expect that the length of the name and value is really irrelevant, but it can't hurt to stress the size limits a bit")),
	(new posix_string'("LAST_NAME"), new posix_string'("xxx")));

    procedure add_vals(vals: env_vals; env: in out environment) is
    begin
	for i in vals'range loop
	    set_environment_variable(vals(i).name.all, vals(i).value.all, env);
	end loop;
    end add_vals;

    procedure check_vals(vals: env_vals; env: in out environment) is
    begin
	for i in vals'range loop
	    if not is_environment_variable(vals(i).name.all, env) then
		fail(to_string(vals(i).name.all) & " missing from environment");
	    end if;
	    require(environment_value_of(vals(i).name.all, env) = vals(i).value.all);
	end loop;
    end check_vals;

    procedure print_val(name: posix_string; value: posix_string; quit: in out boolean) is
    begin
	put(to_string(name)); put("="); put(to_string(value)); nl;
	require(name'first = 1);
	require(value'first = 1);
    end print_val;

    procedure print_env is new for_every_environment_variable(print_val);


    env1, env2, env3: environment;
    initial_env: environment;
begin
    -- First verify basic sanity.
    test("E10");
    require(environment_value_of("ABC") = "abc");
    test("E11");
    require(environment_value_of("undefined_variable") = "");
    test("E12");
    require(environment_value_of("undefined_variable", "XXX") = "XXX");
    test("E13");
    require(is_environment_variable("ABC"));
    test("E14");
    require(not is_environment_variable("undefined_variable"));

    test("E20");
    require(length(env1) = 0);
    test("E21");
    require(not is_environment_variable("ABC", env1));
    test("E22");
    add_vals(data, env1);
    test("E23");
    require(length(env1) = data'length);
    check_vals(data, env1);
    test("E24");
    copy_environment(env1, env2);
    require(length(env2) = data'length);
    check_vals(data, env2);
    test("E25");
    require(is_environment_variable("ABCDEFG", env1));	-- DELETE THIS
    delete_environment_variable("ABCDEFG", env1);
    test("E26");
    require(length(env1) = data'length - 1);
    require(not is_environment_variable("ABCDEFG", env1));
    require(environment_value_of("ABCDEFG", env1, "X") = "X");
    test("E27");
    delete_environment_variable("ABCDEFG", env1);
    require(length(env1) = data'length - 1);
    test("E28");
    add_vals(data, env1);
    require(length(env1) = data'length);
    check_vals(data, env1);
    test("E29");
    clear_environment(env1);
    require(length(env1) = 0);
    require(not is_environment_variable("ABCDEFG", env1));
    test("E40");
    copy_from_current_environment(initial_env);
    test("E41");
    require(is_environment_variable("ABC", initial_env));
    require(environment_value_of("ABC", initial_env) = "abc");
    test("E42");
    add_vals(data, env1);
    test("E44");
    copy_environment(initial_env, env1);
    test("E45");
    require(environment_value_of("ABC", env1) = "abc");
    require(not is_environment_variable("ABCDEFG", env1));
    test("E46");
    add_vals(data, env1);
    check_vals(data, env1);
    require(environment_value_of("ABC", env1) = "abc");
    test("E50");
    clear_environment;
    require(posix_process_environment.length = 0);
    test("E51");
    add_vals(data, current_environment);
    require(length(current_environment) = data'length);
    check_vals(data, current_environment);
    test("E52");
    set_environment_variable("AAA", "BBB");
    require(environment_value_of("AAA") = "BBB");
    test("E53");
    require(is_environment_variable(SKXSJL));
    delete_environment_variable(SKXSJL);
    require(not is_environment_variable(SKXSJL));
    test("E54");
    declare
	found: boolean := false;
	procedure check_ABCDEFG(name: posix_string; value: posix_string;
			quit: in out boolean) is
	begin
	    if found then
		fail("QUIT didn't work");
	    end if;
	    if name = "ABCDEFG" then
		found := true;
		quit := true;
	    end if;
	end check_ABCDEFG;
	procedure check is new
		for_every_current_environment_variable(check_ABCDEFG);
    begin
	check;
	require(found);
    end;
    test("E60");
    begin
	if environment_value_of("a=b") = "" then
	    fail;
	end if;
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("E61");
    declare
	NUL: posix_character := posix_character'val(0);
    begin
	set_environment_variable("a" & NUL, "xxx");
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("E62");
    begin
	delete_environment_variable("a=b");
	except;
    exception
	when posix_error =>
	    check_error(invalid_argument);
    end;
    test("E63");
    begin
	delete_environment_variable("");
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("E70");
    copy_to_current_environment(initial_env);
    require(environment_value_of("ABC") = "abc");
    require(posix_process_environment.length = length(initial_env));
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end environ;


-- Test stuff in the posix package.
procedure posix_pkg is
    s1: string := "hello, world";
    has_nul: posix_string := "hello" & posix_character'val(0);
begin
    test("X10");
    require(to_string(to_posix_string(s1)) = s1);
    test("X12");
    require(is_filename("hello!"));
    test("X13");
    require(not is_filename("a/b"));		-- contains a slash
    test("X14");
    require(is_portable_filename("hello"));
    test("X15");
    require(not is_portable_filename("hello!"));	-- contains an '!'
    test("X16");
    require(is_pathname("a/b"));
    test("X17");
    require(not is_pathname(has_nul));
    test("X18");
    require(is_portable_pathname("a/b"));
    test("X19");
    require(not is_portable_pathname("~/b"));

    -- String lists not tested yet.
    -- Option sets tested elsewhere.

    test("X40");
    set_error_code(too_many_links);
    check_error(too_many_links);
    test("X41");
    set_error_code(no_error);
    check_error(no_error);
    test("X42");
    require(is_posix_error(File_Exists));
    test("X43");
    require(image(File_Exists) = "FILE_EXISTS", "image(File_Exists)");
    require(image(error_code(7142)) = "Error 7142", "image(error_code(7142))");
    test("X44");
    if error_string(File_Exists) /= "File exists" then
	fail("error_string(File_Exists) = " & error_string(File_Exists));
    end if;
    test("X45");
    put("System name = ");
    put(to_string(system_name));
    nl;
    test("X46");
    declare
	got_name: posix_string := node_name;
	expect: posix_string := environment_value_of("NODE_NAME");
    begin
	put("Node name = "); put(to_string(got_name)); nl;
	require(got_name = expect);
    end;
    test("X47");
    put("Operating system release = ");
    put(to_string(release));
    nl;
    test("X48");
    put("Operating system version = ");
    put(to_string(version));
    nl;
    test("X49");
    put("Machine type = ");
    put(to_string(machine));
    nl;
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end posix_pkg;

procedure proc_ident is
    uid: user_id;
    gid: group_id;
begin
    -- Validity of user and group IDs are further tested as part of the
    -- test of get_file_status.
    test("I10");
    uid := get_real_user_id;
    put("uid = "); put(image(uid)); nl;
    test("I11");
    gid := get_real_group_id;
    put("gid = "); put(image(gid)); nl;
    test("I12");
    require(uid = get_effective_user_id);
    test("I13");
    require(gid = get_effective_group_id);
    test("114");
    declare
	i: string := image(uid);
    begin
	require(i'first = 1);
	require(value(i) = uid);
    end;
    test("115");
    declare
	i: string := image(gid);
    begin
	require(i'first = 1);
	require(value(i) = gid);
    end;
    test("I20");
    set_user_id(uid);
    test("I21");
    set_group_id(gid);
    test("I22");
    begin
	set_user_id(value("0"));
	except;
    exception
	when x: posix_error =>
	    check_error(operation_not_permitted, x);
    end;
    test("I23");
    declare
	newgid: group_id;
    begin
	-- use "image" and "value" to define newgid := gid + 1;
	newgid := value(integer'image(integer'value(image(gid)) + 1));
	set_group_id(newgid);
	except;
    exception
	when x: posix_error =>
	    check_error(operation_not_permitted, x);
    end;
    test("I25");
    declare
	list: group_list := get_groups;
    begin
	require(list'first = 1);
	put("Groups: ");
	for i in list'range loop
	    put(' ');
	    put(image(list(i)));
	end loop;
	nl;
    end;
    -- Testing of process ids and process groups deferred until the testing
    -- of process primitives.
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end proc_ident;


-- Check package posix_process_permissions.
procedure permiss is
    perm: permission_set;
    oldperm: permission_set;
begin
    test("A10");
    perm := get_allowed_process_permissions;
    put("Permissions:  "); put_perm(perm); nl;
    require(perm = initial_umask);
    test("A11");
    perm := (others => false);		-- clear perm, just in case
    set_allowed_process_permissions(access_permission_set, perm);
    require(perm = initial_umask);
    test("A12");
    set_allowed_process_permissions(initial_umask);
    perm := get_allowed_process_permissions;
    require(perm = initial_umask);
    test("A15");
    require(access_permission_set = (owner_permission_set
		or group_permission_set or others_permission_set));
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end permiss;


-- Utility routines for checking the posix_file_status package.

package file_status_check is
    type file_type is new string(1..5);
    procedure check_stat(s: status; ftype: file_type);
end file_status_check;

use file_status_check;

package body file_status_check is
    procedure check_stat(s: status; ftype: file_type) is
	got_type: integer := 0;
	ft: file_type;
	now: time;
    begin
	if is_regular_file(s) then
	    ft := "IFREG";
	    got_type := got_type + 1;
	end if;
	if is_directory(s) then
	    ft := "IFDIR";
	    got_type := got_type + 1;
	end if;
	if is_character_special_file(s) then
	    ft := "IFCHR";
	    got_type := got_type + 1;
	end if;
	if is_block_special_file(s) then
	    ft := "IFBLK";
	    got_type := got_type + 1;
	end if;
	if is_fifo(s) then
	    ft := "IFIFO";
	    got_type := got_type + 1;
	end if;
	if is_symbolic_link(s) then	-- extension to POSIX
	    ft := "IFLNK";
	    got_type := got_type + 1;
	end if;
	if is_socket(s) then		-- extension to POSIX
	    ft := "IFSOC";
	    got_type := got_type + 1;
	end if;
	if got_type /= 1 then
	    fail("check_type: got_type /= 1");
	elsif ftype /= ft then
	    put("file type is "); put(string(ft)); nl;
	    fail("check_type: incorrect file type");
	end if;
	now := clock;
	-- put("now "); put_time(now); nl;
	if last_access_time_of(s) > now then
	    put("access time "); put_time(last_access_time_of(s));
	    put(", now "); put_time(now); nl;
	    fail;
	end if;
	if last_modification_time_of(s) > now then
	    put("modification time ");
	    put_time(last_modification_time_of(s));
	    put(", now "); put_time(now); nl;
	    fail;
	end if;
	if last_status_change_time_of(s) > now then
	    put("status change time ");
	    put_time(last_status_change_time_of(s));
	    put(", now "); put_time(now); nl;
	    fail;
	end if;
	put("check_stat done"); nl;
    end check_stat;
end file_status_check;


-- Check the posix_file_status package when getting file status from
-- a names file.  (Getting file status from a file descriptor will
-- be tested as part of the testing of posix_file_io.)
procedure file_stat is
    my_uid: constant user_id := get_real_user_id;
    my_gid: constant group_id := get_real_group_id;
    st: status;
    dev: device_id;
    ino: file_id;
    current_time: time;
    size: io_count;

    test_file_perm: constant permission_set :=
		(owner_read | owner_write | owner_execute
			| set_user_id => true,
		 others => false);
begin
    test("F10");
    st := get_file_status("test_file");
    test("F11");
    check_stat(st, "IFREG");
    dev := device_id_of(st);
    ino := file_id_of(st);
    require(permission_set_of(st) = test_file_perm);
    require(link_count_of(st) = 1);
    require(owner_of(st) = my_uid, "Bad uid");
    require(group_of(st) = my_gid, "Bad gid");
    put("size ="); put(io_count'image(size_of(st))); nl;
    require(size_of(st) = 6);
    test("F12");
    -- Test versions that return ada.calendar.time instead of posix_time:
    require(last_access_time_of(st) =
		to_posix_time(last_access_time_of(st)));
    require(last_modification_time_of(st) =
		to_posix_time(last_modification_time_of(st)));
    require(last_status_change_time_of(st) =
		to_posix_time(last_status_change_time_of(st)));
    test("F13");
    st := get_file_status(".");
    test("F14");
    check_stat(st, "IFDIR");
    begin
	size := size_of(st);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    require(dev /= device_id_of(st) or ino /= file_id_of(st));
    test("F15");
    st := get_file_status("/dev/tty");
    test("F16");
    check_stat(st, "IFCHR");
    test("F17");
    begin
	st := get_file_status(environment_value_of("BLOCK_DEV"));
	check_stat(st, "IFBLK");
    exception
	when posix_error =>
	    fail("Could not get status of BLOCK_DEV");
    end;
    test("F18");
    begin
	st := get_file_status("nonexistent_file");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end file_stat;


-- Test posix_files package.
procedure files is
    st: status;
    st1, st2: status;
    my_uid: constant user_id := get_real_user_id;
    my_gid: constant group_id := get_real_group_id;
begin
    test("F20");
    create_fifo("fifo", access_permission_set);
    test("F21");
    st := get_file_status("fifo");
    check_stat(st, "IFIFO");
    require(permission_set_of(st) = initial_umask);
    test("F22");
    begin
	create_fifo("fifo", owner_permission_set);
	except;
    exception
	when x: posix_error =>
	    check_error(file_exists, x);
    end;
    test("F23");
    unlink("fifo");
    begin
	st := get_file_status("fifo");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;

    test("F30");
    create_directory("dir", access_permission_set);
    test("F31");
    st := get_file_status("dir");
    check_stat(st, "IFDIR");
    if permission_set_of(st) /= initial_umask then
	put("permission_set_of(st) = "); put_perm(permission_set_of(st)); nl;
	put("initial_umask = "); put_perm(initial_umask); nl;
	fail("permission_set incorrect");
    end if;
    test("F32");
    begin
	create_directory("dir", owner_permission_set);
	except;
    exception
	when x: posix_error =>
	    check_error(file_exists, x);
    end;
    test("F33");
    create_fifo("dir/fifo", access_permission_set);
    test("F34");
    begin
	remove_directory("dir");
	except;
    exception
	when posix_error =>
	    if get_error_code = file_exists then
		-- POSIX.5 says this is legal but discouraged.
		put_line("Error File_Exists used for non-empty directory.");
	    else
		check_error(directory_not_empty);
	    end if;
    end;
    test("F35");
    unlink("dir/fifo");
    test("F36");
    remove_directory("dir");
    begin
	st := get_file_status("dir");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;

    test("F40");
    require(is_file("test_file"));
    test("F41");
    require(not is_file("."));
    test("F42");
    require(not is_file("nonexistent_file"));
    test("F44");
    require(is_directory("."));
    test("F45");
    require(not is_directory("test_file"));
    test("F46");
    require(not is_directory("nonexistent_file"));
    test("F47");
    create_fifo("fifo", access_permission_set);
    require(is_fifo("fifo"));
    unlink("fifo");
    test("F48");
    require(not is_fifo("test_file"));
    test("F49");
    require(not is_fifo("nonexistent_file"));
    test("F50");
    require(is_character_special_file("/dev/tty"));
    test("F51");
    require(not is_character_special_file("test_file"));
    test("F52");
    require(not is_character_special_file("nonexistent_file"));
    test("F53");
    require(is_block_special_file(environment_value_of("BLOCK_DEV")));
    test("F54");
    require(not is_block_special_file("test_file"));
    test("F55");
    require(not is_block_special_file("nonexistent_file"));

    test("F60");
    require(is_file_present("test_file"));
    test("F61");
    require(is_file_present("."));
    test("F62");
    require(not is_file_present("nonexistent_file"));
    test("F63");
    require(existence("test_file") = no_error);
    test("F64");
    require(existence("nonexistent_file") = no_such_file_or_directory);

    test("F70");
    link("test_file", "xxx");
    st1 := get_file_status("test_file");
    st2 := get_file_status("xxx");
    require(device_id_of(st1) = device_id_of(st2));
    require(file_id_of(st1) = file_id_of(st2));
    test("F71");
    create_fifo("fifo", access_permission_set);
    begin
	link("test_file", "fifo");
	except;
    exception
	when x: posix_error =>
	    check_error(file_exists, x);
    end;
    test("F72");
    rename("fifo", "zzz");
    require(not is_file_present("fifo"));
    st := get_file_status("zzz");
    check_stat(st, "IFIFO");
    test("F73");
    rename("xxx", "zzz");
    require(not is_file_present("xxx"));
    st := get_file_status("zzz");
    check_stat(st, "IFREG");
    test("F74");
    begin
	rename("nonexistent_file", "zzz");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;
    require(is_file_present("zzz"));
    test("F75");
    rename("zzz", "test_file");
    if not is_file_present("test_file") then
	fail("test_file vanished!");
    elsif is_file_present("zzz") then
	put("Rename fails if both arguments are links to the same file.");
	new_line;
	put("Unfortunately this bug appeared in the original Berkeley implementation");
	new_line;
	put("and POSIX *requires* POSIX compliant systems to reproduce this bug(!)");
	new_line(2);
	flush;
	unlink("zzz");
    else
	put("Rename works even if both arguments are links to the same file.");
	new_line;
	put("Congratulations!  Note, however, that the Berkeley rename system call");
	new_line;
	put("contained a bug in this case, and POSIX thinks you should clone this bug.");
	new_line(2);
	flush;
    end if;

    test("F80");
    declare
	saw_test_file: boolean := false;
	procedure process_file(d: directory_entry; quit: in out boolean) is
	    name: constant posix_string := filename_of(d);
	begin
	    require(name'first = 1);
	    put(to_string(name)); nl;
	    if name = "test_file" then
		require(saw_test_file = false);
		saw_test_file := true;
	    else
		require(name = "." or name = "..");
	    end if;
	end process_file;
	procedure iterate is new for_every_directory_entry(process_file);
    begin
	iterate(".");
    end;
    test("F81");
    declare
	first_call: boolean := true;
	procedure process_file(d: directory_entry; quit: in out boolean) is
	begin
	    require(first_call = true);
	    first_call := false;
	    quit := true;
	end process_file;
	procedure iterate is new for_every_directory_entry(process_file);
    begin
	iterate(".");
	first_call := true;
	iterate("..");
    end;
    test("F82");
    declare
	my_exception: exception;
	first_call: boolean := true;
	procedure process_file(d: directory_entry; quit: in out boolean) is
	begin
	    require(first_call = true);
	    first_call := false;
	    raise my_exception;
	end process_file;
	procedure iterate is new for_every_directory_entry(process_file);
    begin
	iterate(".");
	except;
    exception
	when my_exception => null;
    end;
    test("F100");
    create_directory("dir", access_permission_set);
    change_owner_and_group("test_file", my_uid, my_gid);
	-- File should be unchanged.
    test("F101");
    begin
	change_owner_and_group("nonexistent_file", my_uid, my_gid);
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;

    test("F102");
    change_permissions("dir", owner_permission_set);
    st := get_file_status("dir");
    check_stat(st, "IFDIR");
    require(permission_set_of(st) = owner_permission_set);
    test("F103");
    begin
	change_permissions("nonexistent_file", owner_permission_set);
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;

    test("F104");
    declare
	access_time: posix_time;
	mod_time: posix_time;
	d_3600: duration := 3600.0;
	d_1800: duration := 1800.0;
    begin
	access_time := to_posix_time(clock) - d_3600;
	mod_time := access_time + d_1800;
	set_file_times("dir", access_time, mod_time);
	st := get_file_status("dir");
	if last_access_time_of(st) /= access_time then
	    put("Set access time to:  "); put_posix_time(access_time); nl;
	    put("But access time is:  ");
	    put_posix_time(last_access_time_of(st)); nl;
	end if;
	require(last_access_time_of(st) = access_time);
	require(last_modification_time_of(st) = mod_time);
    end;
    test("F105");
    declare
	before: posix_time := clock;
	after: posix_time;
    begin
	set_file_times("dir");
	after := clock;
	st := get_file_status("dir");
	if before > last_access_time_of(st)
		or after < last_access_time_of(st) then
	    fail("set_file_times didn't set access time to current time");
	    put("Before:       "); put_posix_time(before); nl;
	    put("Access time:  "); put_posix_time(last_access_time_of(st)); nl;
	    put("After:        "); put_posix_time(after); nl;
	end if;
	require(before <= last_modification_time_of(st));
	require(after >= last_modification_time_of(st));
    end;
    test("F106");
    declare
	access_time: posix_time;
	mod_time: posix_time;
    begin
	access_time := to_posix_time(clock);
	mod_time := to_posix_time(clock);
	set_file_times("nonexistent_file", access_time, mod_time);
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;
    test("F107");
    begin
	set_file_times("nonexistent_file");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;
    remove_directory("dir");

    -- Test symbolic links.
    test("F120");
    create_symbolic_link("test_file", "link");
    test("F121");
    st1 := get_file_status("link");
    st2 := get_file_status("test_file");
    require(st1 = st2);
    test("F122");
    begin
	create_symbolic_link("test_file", "link");
	except;
    exception
	when x: posix_error =>
	    check_error(file_exists, x);
    end;
    test("F123");
    st1 := get_link_status("link");
    check_stat(st1, "IFLNK");
    test("F124");
    require(size_of(st1) = 9);
    test("F125");
    begin
	st1 := get_link_status("nonexistent_file");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;
    test("F126");
    st1 := get_link_status("test_file");
    st2 := get_file_status("test_file");
    require(st1 = st2);
    test("F127");
    require(is_symbolic_link("link"));
    test("F128");
    require(is_file("link"));
    test("F129");
    require(not is_symbolic_link("test_file"));
    test("F130");
    require(not is_symbolic_link("."));
    test("F131");
    require(not is_symbolic_link("nonexistent_file"));
    test("F132");
    declare
	name: posix_string := read_symbolic_link("link");
    begin
	require(name = "test_file");
    end;
    test("F133");
    begin
	declare
	    name: posix_string := read_symbolic_link("nonexistent_file");
	begin
	    except;
	end;
    exception
	when x: posix_error =>
	    put_line(ada.exceptions.exception_message(x));
	    check_error(no_such_file_or_directory, x);
    end;
    test("F134");
    begin
	declare
	    name: posix_string := read_symbolic_link("test_file");
	begin
	    except;
	end;
    exception
	when x: posix_error =>
	    put_line(ada.exceptions.exception_message(x));
	    check_error(invalid_argument, x);
    end;
    test("F135");
    require(not is_socket("test_file"));
    test("F136");
    require(not is_socket("nonexistent_file"));
    test("F137");
    st1 := get_link_status(".");
    require(not is_socket(st1));
    test("F138");
    unlink("link");
    require(is_file_present("test_file"));
    require(not is_file_present("link"));
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end files;


-- Test current working directory stuff in posix_process_environment.
procedure cwdir is
    type posix_string_ptr is access posix_string;
    cwd: posix_string_ptr;
begin
    test("W10");
    cwd := new posix_string'(get_working_directory);
    put("cwd = "); put(to_string(cwd.all)); nl;
    require(cwd.all'first = 1);
    require(cwd.all = environment_value_of("CWD"));
    test("W11");
    change_working_directory("..");
    require(is_directory("testdir"));
    test("W12");
    change_working_directory("testdir");
    require(not is_directory("testdir"));
    test("W13");
    begin
	change_working_directory("nonexistent_directory");
	except;
    exception
	when x: posix_error =>
	    check_error(no_such_file_or_directory, x);
    end;
    test("W14");
    declare
	newcwd: posix_string := get_working_directory;
    begin
	require(newcwd'first = 1);
	require(newcwd = cwd.all);
    end;
    -- Don't bother to free cwd.  Since this routine is only called once
    -- the storage leak hardly matters.
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end cwdir;


-- Test option sets.
procedure sets is
    o1, o2: open_option_set;
begin
    test("S10");
    require(o1 = empty_set);
    require(o2 = empty_set);
    test("S11");
    require(o1 + o2 = empty_set);
    test("S12");
    require(o1 - o2 = empty_set);
    test("S13");
    o1 := posix_io.append + non_blocking;
    o2 := posix_io.append + truncate;
    require(o1 + o2 = posix_io.append + non_blocking + truncate);
    test("S14");
    require(o1 - o2 = non_blocking);
end sets;


-- procedure file_io was here, but was moved to a separate file to
-- make this file smaller.


-- Test posix_process_primitives and posix_unsafe_process_primitives.

-- The pause procedure can be rewritten as an infinite loop if necessary.
procedure c_pause;
pragma import(C, c_pause, "pause");

procedure pause is
begin
    loop
	c_pause;
    end loop;
end pause;


-- Check the exit status from running the program named "child".
procedure check_child(status: termination_status; exitstat: exit_status) is
    e: exit_status;
begin
    if not status_available(status) then
	fail("status not available");
    elsif termination_cause_of(status) /= exited then
	if termination_cause_of(status) = terminated_by_signal then
	    put("Signal is "); put(integer(termination_signal_of(status))); nl;
	end if;
	fail("did not exit");
    else
	e := exit_status_of(status);
	if e > 0 and e <= 39 then
	    test_pkg.error_count := test_pkg.error_count + integer(e);
	elsif e /= exitstat then
	    fail("unexpected exit status" & exit_status'image(e));
	end if;
    end if;
end check_child;




procedure proc is
    pid: process_id;
    status: termination_status;
    sig: signal;
    exitstat: exit_status;
    parent: process_id;
    pgrp: process_group_id;
    args: posix_string_list;
    env: environment;
    tem: process_template;
    mask: signal_set;
begin
    test("P10");
    parent := get_process_id;
    put("pid = "); put(image(parent)); nl;
    test("P11");
    require(not status_available(status));
    test("P20");
    pid := fork;
    if pid = null_process_id then
	exit_process(27);
    end if;
    test("P21");
    wait_for_child_process(status, true, false);
    test("P22");
    require(status_available(status), "status not available");
    test("P23");
    require(process_id_of(status) = pid, "wrong pid");
    test("P24");
    require(termination_cause_of(status) = exited);
    test("P25");
    require(exit_status_of(status) = 27);
    test("P26");
    begin
	sig := termination_signal_of(status);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P27");
    begin
	sig := stopping_signal_of(status);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P30");
    pid := fork;
    if pid = null_process_id then
	pause;
    end if;
    test("P31");
    send_signal(pid, signal_stop);
    test("P32");
    wait_for_child_process(status, true, true);
    test("P33");
    require(status_available(status), "status not available");
    require(process_id_of(status) = pid, "wrong pid");
    require(termination_cause_of(status) = stopped_by_signal);
    require(stopping_signal_of(status) = signal_stop);
    test("P34");
    begin
	exitstat := exit_status_of(status);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P35");
    send_signal(pid, signal_continue);
    send_signal(pid, signal_terminate);
    test("P36");
    wait_for_child_process(status, true, true);
    test("P37");
    require(status_available(status), "status not available");
    require(process_id_of(status) = pid, "wrong pid");
    require(termination_cause_of(status) = terminated_by_signal);
    require(termination_signal_of(status) = signal_terminate);

    declare
	child: array (1 .. 500) of process_id;
	num_children: natural := 0;
	children_limit: natural;
    begin
	test("P40a");
	-- We subtract two to account for runtest and ourself.
	children_limit := child_processes_maximum - 2;
	test("P40b");
	loop
	    begin
		pid := fork;
		if pid = null_process_id then
		    pause;
		end if;
	    exception
		when posix_error =>
		    exit;
	    end;
	    num_children := num_children + 1;
	    child(num_children) := pid;
	    if num_children > children_limit then
		fail("Fork didn't enforce child_processes limit");
		exit;
	    elsif num_children = child'last then
		-- This can happen on Solaris.
		fail("Test not completed; CHILD_MAX is too large.");
		exit;
	    end if;
	end loop;
	test("P41");
	for i in 1 .. num_children loop
	    send_signal(child(i), signal_terminate);
	end loop;
	test("P42");
	for i in 1 .. num_children loop
	    -- If masked_signals is set to rts_signals,
	    -- the following call tends to core dump under Solaris.
	    wait_for_child_process(status, child(i));
	end loop;
	put(num_children); put(" children"); nl;
    end;
    test("P50");
    pid := fork;
    if pid = null_process_id then
	pause;
    end if;
    test("P51");
    wait_for_child_process(status, false);
    test("P52");
    require(not status_available(status));
    test("P53");
    begin
	put(termination_cause'image(termination_cause_of(status)));
	nl;
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P54");
    send_signal(pid, signal_terminate);
    wait_for_child_process(status, true);
    require(status_available(status));
    test("P55");
    begin
	wait_for_child_process(status, true);
	except;
    exception
	when x: posix_error =>
	    check_error(no_child_process, x);
    end;

    -- Test process identification.
    test("P60");
    pid := fork;
    if pid = null_process_id then
	if get_parent_process_id /= parent then
	    exit_process(10);
	end if;
	send_signal(get_process_id, signal_terminate);
	exit_process(11);
    end if;
    wait_for_child_process(status);
    if termination_cause_of(status) /= terminated_by_signal then
	fail("process not killed");
	if termination_cause_of(status) = exited then
	    put("exit status = "); put(exit_status_of(status)); nl;
	end if;
    else
	require(termination_signal_of(status) = signal_terminate);
    end if;
    test("P61");
    pgrp := get_process_group_id;
    test("P62");
    pid := fork;
    if pid = null_process_id then
	begin
	    create_session(pgrp);
	    if get_process_group_id /= pgrp then
		exit_process(10);
	    end if;
	    declare
		junk: process_group_id;
	    begin
		create_session(junk);
		exit_process(11);
	    exception
		when posix_error =>
		    if get_error_code /= operation_not_permitted then
			exit_process(12);
		    end if;
	    end;
	    send_signal(get_process_group_id, signal_terminate);
	    exit_process(13);
	exception
	    when posix_error =>
		exit_process(14);
	end;
    end if;
    wait_for_child_process(status);
    if termination_cause_of(status) /= terminated_by_signal then
	fail("process not killed");
	if termination_cause_of(status) = exited then
	    put("exit status = "); put(exit_status_of(status)); nl;
	end if;
    else
	require(termination_signal_of(status) = signal_terminate);
    end if;
    test("P70");
    begin
	set_keep_effective_ids(tem);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P71");
    begin
	close_template(tem);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P72");
    begin
	start_process(pid, "../child", tem, args);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("P73");
    open_template(tem);
    test("P74");
    set_keep_effective_ids(tem);
    test("P75");
    require(length(args) = 0);
    posix.append(args, "child");
    posix.append(args, "2");
    posix.append(args, "ABC=abc");
    posix.append(args, "unblocked SIGNAL_USER_1");
    test("P76");
    start_process(pid, "../child", tem, args);
    test("P77");
    wait_for_child_process(status, pid);
    test("P78");
    check_child(status, 0);
    test("P80");
    open_template(tem);
    test("P81");
    add_signal(mask, signal_user_1);
    set_signal_mask(tem, mask);
    test("P82");
    make_empty(args);
    posix.append(args, "child");
    posix.append(args, "2");
    posix.append(args, "ABC=abc");
    posix.append(args, "blocked SIGNAL_USER_1");
    test("P83");
    start_process(pid, "../child", tem, args);
    wait_for_child_process(status, pid);
    check_child(status, 0);
    test("P90");
    open_template(tem);
    test("P91");
    set_keep_effective_ids(tem);
    set_file_action_to_open(tem, 3, "test_file");
    set_file_action_to_duplicate(tem, 5, 3);
    set_file_action_to_close(tem, 0);
    test("P92");
    make_empty(args);
    posix.append(args, "child");
    posix.append(args, "4");
    posix.append(args, "ABC=abc");
    posix.append(args, "0 closed");
    posix.append(args, "3=test_file");
    posix.append(args, "5=test_file");
    test("P93");
    start_process_search(pid, "../child", tem, args);
    wait_for_child_process(status, pid);
    check_child(status, 0);
    test("P95");
    copy_from_current_environment(env);
    delete_environment_variable("ABC", env);
    set_environment_variable("XXX", "xyz", env);
    test("P96");
    set_file_action_to_close(tem, 3);
    test("P97");
    make_empty(args);
    posix.append(args, "child");
    posix.append(args, "5");
    posix.append(args, "ABC undefined");
    posix.append(args, "XXX=xyz");
    posix.append(args, "0 closed");
    posix.append(args, "3 closed");
    posix.append(args, "5=test_file");
    test("P98");
    start_process(pid, "../child", tem, env, args);
    wait_for_child_process(status, pid);
    check_child(status, 0);
    test("P100");
    pid := fork;
    if pid = null_process_id then
	create_process_group(get_process_id, pgrp);
	if get_process_group_id /= pgrp then
	    exit_process(10);
	end if;
	send_signal(signal_terminate);
	exit_process(11);
    end if;
    wait_for_child_process(status);
    if termination_cause_of(status) /= terminated_by_signal then
	fail("process not killed");
	if termination_cause_of(status) = exited then
	    put("exit status = "); put(exit_status_of(status)); nl;
	end if;
    else
	require(termination_signal_of(status) = signal_terminate);
    end if;
    test("P102");
    pid := fork;
    if pid = null_process_id then
	make_empty(args);
	posix.append(args, "child");
	posix.append(args, "2");
	posix.append(args, "ABC=abc");
	posix.append(args, "0 closed");
	if is_open(0) then
	    close(0);
	end if;
	exec("../child", args);
	exit_process(99);
    end if;
    wait_for_child_process(status, pid);
    check_child(status, 0);
    -- Note that start_process uses exec internally, which provides
    -- additional testing of exec.

    test("P104");
    open_template(tem);
    set_keep_effective_ids(tem);
    start_process_search(pid, "nonexistent file--we hope", tem, env, args);
    wait_for_child_process(status, pid);
    check_child(status, failed_creation_exit);
    test("P106");
    open_template(tem);
    set_keep_effective_ids(tem);
    set_file_action_to_open(tem, 3, "nonexistent_file");
    make_empty(args);
    posix.append(args, "child");
    posix.append(args, "1");
    posix.append(args, "ABC=abc");
    start_process(pid, "../child", tem, env, args);
    wait_for_child_process(status, pid);
    check_child(status, failed_creation_exit);

    test("P110");
    close_template(tem);
    test("P111");
    clear_environment(env);
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end proc;


-- Test POSIX file locking.
procedure posixlock is
    use posix_file_locking;
    test_fd: file_descriptor;	-- file descriptor for test_file.
    lock1: file_lock;
    locker: process_id;
    child: process_id;
    read_fd, write_fd: file_descriptor;
    last: io_count;
    buf: posix_string(1 .. 5);
    status: termination_status;
    pid: process_id;
begin
    test("PL10");
    test_fd := open("test_file", read_write);
    create_pipe(read_fd, write_fd);
    test("PL11");
    child := fork;
    if child = null_process_id then
	set_lock(test_fd, (whole_file => false, lock => read_lock,
				starting_point => from_beginning,
				start => 3, length => 2));
	close(read_fd);
	write(write_fd, "ready", last, no_signals);
	close(write_fd);
	pause;
    end if;
    test("PL12");
    close(write_fd);
    read(read_fd, buf, last, no_signals);
    require(buf(1 .. integer(last)) = "ready");
    -- Now the file has a read lock.
    test("PL20");
    set_lock(test_fd, (false, write_lock, from_beginning, 0, 3));
    test("PL21");
    set_lock(test_fd, (false, write_lock, from_end_of_file, -6, 3));
    test("PL22");
    set_lock(test_fd, (false, write_lock, from_beginning, 5, 1));
    test("PL23");
    set_lock(test_fd, (false, write_lock, from_end_of_file, -1, 1));
    test("PL24");
    begin
	set_lock(test_fd, (false, write_lock, from_beginning, 3, 2));
	except;
    exception
	when x: posix_error =>
	    check_error(resource_temporarily_unavailable, x);
    end;
    test("PL25");
    begin
	set_lock(test_fd, (false, write_lock, from_end_of_file, -3, 2));
	except;
    exception
	when x: posix_error =>
	    check_error(resource_temporarily_unavailable, x);
    end;
    test("PL26");
    get_lock(test_fd, (false, write_lock, from_end_of_file, -3, 1),
		lock1, locker);
    if locker = null_process_id then
	fail("No lock found");
    else
	require(locker = child, "Wrong locking process");
	require(lock1.lock = read_lock, "Not a read lock");
	if lock1.whole_file then
	    fail("whole file locked");
	elsif lock1.starting_point /= from_beginning
	   or lock1.start /= 3
	   or lock1.length /= 2 then
	    put("lock: "); put(position'image(lock1.starting_point));
	    put(" <"); put(lock1.start); put(","); put(lock1.length);
	    put(">"); nl;
	    fail("Incorrect lock found");
	end if;
    end if;
    test("PL27");
    get_lock(test_fd, (false, read_lock, from_end_of_file, -3, 1),
		lock1, locker);
    require(locker = null_process_id);
    test("PL28");
    set_lock(test_fd, (false, read_lock, from_beginning, 3, 2));
    test("PL29");
    set_lock(test_fd, (true, read_lock));
    test("PL30");
    begin
	set_lock(test_fd, (true, write_lock));
	except;
    exception
	when x: posix_error =>
	    check_error(resource_temporarily_unavailable, x);
    end;
    test("PL31");
    set_lock(test_fd, (true, unlock));
    test("PL32");
    send_signal(child, signal_terminate);
    wait_for_child_process(status, child);
    require(termination_cause_of(status) = terminated_by_signal);
    require(termination_signal_of(status) = signal_terminate);
    test("PL40");
    set_lock(test_fd, (true, write_lock));
    test("PL41");
    child := fork;
    if child = null_process_id then
	wait_to_set_lock(test_fd, (true, write_lock));
	exit_process(0);
    end if;
    test("PL42");
    -- Delay a while.
    pid := fork;
    if pid = null_process_id then
	exit_process(0);
    end if;
    wait_for_child_process(status, pid);
    pid := fork;
    if pid = null_process_id then
	exit_process(0);
    end if;
    wait_for_child_process(status, pid);
    test("PL43");
    wait_for_child_process(status, child, block => false);
    require(not status_available(status));
    test("PL44");
    set_lock(test_fd, (true, unlock));
    test("PL45");
    wait_for_child_process(status, child);
    require(termination_cause_of(status) = exited);	
    require(exit_status_of(status) = 0);
    test("PL46");
    close(test_fd);
    close(read_fd);
    test("PL47");
    begin
	get_lock(test_fd, (true, write_lock), lock1, locker);
	except;
    exception
	when x: posix_error =>
	    check_error(bad_file_descriptor, x);
    end;

exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end posixlock;


-- Test Forest file locking.
procedure forestlock is
    use forest_file_locking;
    test_fd: file_descriptor;	-- file descriptor for test_file.
    lock1: file_lock;
    locker: process_id;
    child: process_id;
    read_fd, write_fd: file_descriptor;
    last: io_count;
    buf: posix_string(1 .. 5);
    status: termination_status;
    pid: process_id;
begin
    test("FL10");
    test_fd := open("test_file", read_write);
    create_pipe(read_fd, write_fd);
    test("FL11");
    child := fork;
    if child = null_process_id then
	set_lock(test_fd, (kind => read_lock,
				starting_point => from_beginning,
				start => 3, length => 2));
	close(read_fd);
	write(write_fd, "ready", last, no_signals);
	close(write_fd);
	pause;
    end if;
    test("FL12");
    close(write_fd);
    read(read_fd, buf, last, no_signals);
    require(buf(1 .. integer(last)) = "ready");
    -- Now the file has a read lock.
    test("FL20");
    set_lock(test_fd, (write_lock, from_beginning, 0, 3));
    test("FL21");
    set_lock(test_fd, (write_lock, from_end_of_file, -6, 3));
    test("FL22");
    set_lock(test_fd, (write_lock, from_beginning, 5, 1));
    test("FL23");
    set_lock(test_fd, (write_lock, from_end_of_file, -1, 1));
    test("FL24");
    begin
	set_lock(test_fd, (write_lock, from_beginning, 3, 2));
	except;
    exception
	when x: posix_error =>
	    check_error(resource_temporarily_unavailable, x);
    end;
    test("FL25");
    begin
	set_lock(test_fd, (write_lock, from_end_of_file, -3, 2));
	except;
    exception
	when x: posix_error =>
	    check_error(resource_temporarily_unavailable, x);
    end;
    test("FL26");
    get_lock(test_fd, (write_lock, from_end_of_file, -3, 1),
		lock1, locker);
    if locker = null_process_id then
	fail("No lock found");
    else
	require(locker = child, "Wrong locking process");
	require(lock1.kind = read_lock, "Not a read lock");
	if lock1.starting_point /= from_beginning
	   or lock1.start /= 3
	   or lock1.length /= 2 then
	    put("lock: "); put(position'image(lock1.starting_point));
	    put(" <"); put(lock1.start); put(","); put(lock1.length);
	    put(">"); nl;
	    fail("Incorrect lock found");
	end if;
    end if;
    test("FL27");
    get_lock(test_fd, (read_lock, from_end_of_file, -3, 1),
		lock1, locker);
    require(locker = null_process_id);
    test("FL28");
    set_lock(test_fd, (read_lock, from_beginning, 3, 2));
    test("FL29");
    set_lock(test_fd, (read_lock, from_beginning, 0, infinity));
    test("FL30");
    begin
	set_lock(test_fd, (write_lock, from_beginning, 0, infinity));
	except;
    exception
	when x: posix_error =>
	    check_error(resource_temporarily_unavailable, x);
    end;
    test("FL31");
    set_lock(test_fd, (unlock, from_beginning, 0, infinity));
    test("FL32");
    send_signal(child, signal_terminate);
    wait_for_child_process(status, child);
    require(termination_cause_of(status) = terminated_by_signal);
    require(termination_signal_of(status) = signal_terminate);
    test("FL40");
    set_lock(test_fd, (write_lock, from_beginning, 0, infinity));
    test("FL41");
    child := fork;
    if child = null_process_id then
	wait_to_set_lock(test_fd, (write_lock, from_beginning, 0, infinity));
	exit_process(0);
    end if;
    test("FL42");
    -- Delay a while.
    pid := fork;
    if pid = null_process_id then
	exit_process(0);
    end if;
    wait_for_child_process(status, pid);
    pid := fork;
    if pid = null_process_id then
	exit_process(0);
    end if;
    wait_for_child_process(status, pid);
    test("FL43");
    wait_for_child_process(status, child, block => false);
    require(not status_available(status));
    test("FL44");
    set_lock(test_fd, (unlock, from_beginning, 0, infinity));
    test("FL45");
    wait_for_child_process(status, child);
    require(termination_cause_of(status) = exited);	
    require(exit_status_of(status) = 0);
    test("FL46");
    close(test_fd);
    close(read_fd);
    test("FL47");
    begin
	get_lock(test_fd, (write_lock, from_beginning, 0, infinity), lock1, locker);
	except;
    exception
	when x: posix_error =>
	    check_error(bad_file_descriptor, x);
    end;

exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end forestlock;


procedure user_db is
    item: user_database_item;
    alt_item: user_database_item;
    uid: user_id;
begin
    test("D10");
    begin
	uid := user_id_of(item);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("D11");
    begin
	put_line(to_string(initial_directory_of(item)));
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("D20");
    uid := get_real_user_id;
    item := get_user_database_item(uid);
    test("D21");
    put("user name:  "); put(to_string(user_name_of(item))); nl;
    put("user id:    "); put(image(user_id_of(item))); nl;
    put("group id:   "); put(image(group_id_of(item))); nl;
    put("directory:  "); put(to_string(initial_directory_of(item))); nl;
    put("shell:      "); put(to_string(initial_program_of(item))); nl;
    test("D22");
    alt_item := get_user_database_item(user_name_of(item));
    test("D23");
    require(user_id_of(item) = user_id_of(alt_item), "uid differ");
    require(alt_item = item);
    test("D25");
    begin
	item := get_user_database_item("NoSuchUser");
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);	-- CHECK THIS --
    end;
    test("D26");
    declare
	i: integer;
    begin
	i := 10000;
	loop
	    uid := value(integer'image(i));
	    begin
		item := get_user_database_item(uid);
	    exception
		when x: posix_error =>
		    check_error(invalid_argument, x);	-- CHECK THIS --
		    exit;
	    end;
	    put("uid "); put(i); put(" => ");
	    put(to_string(user_name_of(item))); nl;
	    i := i + 1000;
	end loop;
    end;

exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end user_db;


procedure group_db is
    item: group_database_item;
    alt_item: group_database_item;
    gid: group_id;
    group_list: group_id_list;
begin
    test("D30");
    begin
	gid := group_id_of(item);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("D31");
    begin
	group_list := group_id_list_of(item);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    test("D40");
    gid := get_real_group_id;
    item := get_group_database_item(gid);
    test("D41");
    put("group name: "); put(to_string(group_name_of(item))); nl;
    put("group id:   "); put(image(group_id_of(item))); nl;
    group_list := group_id_list_of(item);
    put("group list: "); put(length(group_list)); put(" items"); nl;
    test("D42");
    declare
	procedure print(id: posix_string; quit: in out boolean) is
	begin
	    put(" "); put(to_string(id));
	end print;
	procedure print_members is new for_every_member(print);
    begin
	put("members of group ");
	put(to_string(group_name_of(item)));
	put(": ");
	print_members(group_list);
	nl;
    end;
    test("D43");
    declare
	procedure print(id: posix_string; quit: in out boolean) is
	begin
	    put(" "); put(to_string(id));
	    quit := true;
	end print;
	procedure print_members is new for_every_member(print);
    begin
	put("first member: ");
	print_members(group_list);
	nl;
    end;
    test("D45");
    alt_item := get_group_database_item(group_name_of(item));
    test("D46");
    require(group_id_of(alt_item) = group_id_of(item), "gids differ");
    require(group_name_of(alt_item) = group_name_of(item), "names differ");
    require(group_id_list_of(alt_item) = group_id_list_of(item),
	"group id lists differ");
    require(alt_item = item, "alt_item /= item");
    test("D48");
    begin
	item := get_group_database_item("NoSuchGroup");
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument,x );	-- CHECK THIS --
    end;
    test("D49");
    declare
	i: integer;
    begin
	i := 10000;
	loop
	    gid := value(integer'image(i));
	    begin
		item := get_group_database_item(gid);
	    exception
		when x: posix_error =>
		    check_error(invalid_argument, x);	-- CHECK THIS --
		    exit;
	    end;
	    put("gid "); put(i); put(" => ");
	    put(to_string(group_name_of(item))); nl;
	    i := i + 1000;
	end loop;
    end;
exception
    when occurrence: posix_error =>
	fail("Unexpected POSIX_Error: "
		& ada.exceptions.exception_message(occurrence));
end group_db;



begin -- main
    put("Test program started.  POSIX Ada Version = ");
    put(integer'(posix_ada_version));
    nl;

    -- run tests --
    basic;
    string_list;
    environ;
    posix_pkg;
    proc_ident;
    signals;	-- Could move lower?
    permiss;
    file_stat;
    files;
    cwdir;
    sets;
    file_io;
    proc;
    posixlock;
    forestlock;
    user_db;
    group_db;
    done;
end main;
