--------------------------------------------------------------------------
--  Copyright (C) 1995 by Kenneth Almquist				--
--									--
--  This program is free software; you can redistribute it and/or	--
--  modify it under the terms of the GNU General Public License as	--
--  published by the Free Software Foundation; either version 2 of	--
--  the License, or (at your option) any later version.			--
--------------------------------------------------------------------------

-- Test terminal functions
with text_io; use text_io;
with test_pkg; use test_pkg;
with posix; use posix;
with posix.more; use posix.more;
with posix_io; use posix_io;
with posix_terminal_functions; use posix_terminal_functions;
with posix_terminal_functions.more; use posix_terminal_functions.more;
procedure terminal is

procedure put is new test_pkg.putint(integer);

procedure tty is
    original_tc: terminal_characteristics;
    tc: terminal_characteristics;
    new_tc: terminal_characteristics;
    modes: terminal_modes_set;
    new_modes: terminal_modes_set;
    test_fd: file_descriptor;
    baud: baud_rate;
    speed: terminal_speed;
begin
    test("T10");
    original_tc := get_terminal_characteristics(posix_io.standard_error);
    test("T11");
    tc := original_tc;
    modes := terminal_modes_of(tc);
    require(modes = terminal_modes_of(original_tc));
    test("T12");
    require(modes(enable_signals) = true, "enable_signals not set");
    test("T13");
    modes(ignore_break) := true;
    modes(mark_parity_errors) := true;
    modes(perform_output_processing) := true;
    modes(echo_kill) := true;
    modes(no_flush) := true;
    define_terminal_modes(tc, modes);
    test("T14");
    set_terminal_characteristics(posix_io.standard_error, tc, after_output);
    test("T15");
    new_tc := get_terminal_characteristics(posix_io.standard_error);
    new_modes := terminal_modes_of(new_tc);
    require(new_modes = modes);
    test("T16");
    modes(ignore_break) := false;
    modes(mark_parity_errors) := false;
    modes(perform_output_processing) := false;
    modes(echo_kill) := false;
    modes(no_flush) := false;
    define_terminal_modes(new_tc, modes);
    set_terminal_characteristics(posix_io.standard_error, new_tc);
    test("T17");
    tc := new_tc;
    new_tc := get_terminal_characteristics(posix_io.standard_error);
    new_modes := terminal_modes_of(new_tc);
    require(new_modes = modes);
    test("T18");
    define_minimum_input_count(tc, 17);
    require(minimum_input_count_of(tc) = 17, "input count wrong");
    define_input_time(tc, 0.5);
    put("input time ="); put(duration'image(input_time_of(tc))); nl;
    require(input_time_of(tc) = 0.5, "input time wrong");
    set_terminal_characteristics(posix_io.standard_error, tc,
				 after_output_and_input);
    test("T19");
    new_tc := get_terminal_characteristics(posix_io.standard_error);
    new_modes := terminal_modes_of(new_tc);
    require(new_modes = modes, "modes incorrect");
    put("count = "); put(minimum_input_count_of(new_tc));
    nl;
    require(minimum_input_count_of(new_tc) = minimum_input_count_of(tc), "count wrong");
    require(input_time_of(new_tc) = input_time_of(tc), "time wrong");
    test("T20");
    declare
	uninitialized: terminal_characteristics;
    begin
	set_terminal_characteristics(posix_io.standard_error, uninitialized);
	except;
    exception
	when posix_error =>
	    check_error(invalid_argument);
    end;
    test("T21");
    begin
	define_minimum_input_count(tc, 30000);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    require(minimum_input_count_of(tc) = 17);
    test("T22");
    begin
	define_input_time(tc, 100.0);
	except;
    exception
	when x: posix_error =>
	    check_error(invalid_argument, x);
    end;
    require(input_time_of(tc) = 0.5);
    test("T23");
    test_fd := open("test_file", read_write);
    begin
	set_terminal_characteristics(test_fd, tc);
	except;
    exception
	when x: posix_error =>
	    check_error(inappropriate_io_control_operation, x);
    end;
    test("T24");
    begin
	tc := get_terminal_characteristics(test_fd);
	except;
    exception
	when x: posix_error =>
	    check_error(inappropriate_io_control_operation, x);
    end;
    close(test_fd);

    -- Test terminal speeds.
    test("T30");
    speed := 300.0;
    speed := (speed + speed) / 300.0;
    -- The following is commented out because the required operators
    -- cannot be renamed under GNAT 2.07.  (Bug fixed in GNAT 3.01.)
    --     require(speed / 2 = speed - 1.0 and 2 * speed /= speed);
    test("T31");
    speed := input_speed_of(original_tc);
    put("Input speed "); put(terminal_speed'image(speed)); nl;
    speed := output_speed_of(original_tc);
    put("Output speed"); put(terminal_speed'image(speed)); nl;
    test("T32");
    tc := original_tc;
    define_input_baud_rate(tc, B300);
    define_output_baud_rate(tc, B1200);
    test("T33");
    baud := input_baud_rate_of(tc);
    require(baud = B300, "input baud rate incorrect");
    baud := output_baud_rate_of(tc);
    require(baud = B1200, "output_baud_rate_incorrect");
    speed := input_speed_of(tc);
    require(speed = 300.0, "input speed incorrect");
    speed := output_speed_of(tc);
    require(speed = 1200.0, "output speed incorrect");
    test("T34");
    define_input_speed(tc, 34567.875);
    require(input_speed_of(tc) = 34567.875);
    define_output_speed(tc, 300.0);
    require(output_speed_of(tc) = 300.0);
    test("T35");
    -- The following call should not change the terminal speeds, because
    -- the implementation does not (presumably) support the requested
    -- input speed.
    begin
	set_terminal_characteristics(posix_io.standard_error, tc);
    exception
	when x: posix_error =>
	    fail("POSIX_Error raised: " & error_string(error_code_of(x)));
    end;
    test("T36");
    tc := get_terminal_characteristics(posix_io.standard_error);
    require(input_speed_of(tc) = input_speed_of(original_tc), "input speed changed");
    require(output_speed_of(tc) = output_speed_of(original_tc), "output speed changed");
    test("T37");
    define_input_speed(tc, 300.0);
    define_output_speed(tc, 300.0);
    set_terminal_characteristics(posix_io.standard_error, tc);
    test("T38");
    tc := get_terminal_characteristics(posix_io.standard_error);
    speed := input_speed_of(tc);
    if speed /= 300.0 then
	fail("input speed is " & terminal_speed'image(speed));
    end if;
    speed := output_speed_of(tc);
    if speed /= 300.0 then
	fail("output speed is " & terminal_speed'image(speed));
    end if;

    -- Restore original terminal modes.
    test("T98");
    set_terminal_characteristics(posix_io.standard_error, original_tc);
    test("T99");
    tc := get_terminal_characteristics(posix_io.standard_error);
    require(tc = original_tc);
exception
    when posix_error =>
	fail("Unexpected POSIX_Error: " & image(get_error_code));
end tty;

begin -- terminal
    put("Test program started."); nl;

    -- run tests --
    tty;
    done;
end terminal;
