/****************************************************************************/
/*                                                                          */
/*                         GNAT COMPILER COMPONENTS                         */
/*                                                                          */
/*                              A - R A I S E                               */
/*                                                                          */
/*                          C Implementation File                           */
/*                                                                          */
/*                            $Revision: 1.17 $                             */
/*                                                                          */
/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
/*                                                                          */
/* GNAT is free software;  you can  redistribute it  and/or modify it under */
/* terms of the  GNU General Public License as published  by the Free Soft- */
/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
/* for  more details.  You should have  received  a copy of the GNU General */
/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
/* MA 02111-1307, USA.                                                      */
/*                                                                          */
/* As a  special  exception,  if you  link  this file  with other  files to */
/* produce an executable,  this file does not by itself cause the resulting */
/* executable to be covered by the GNU General Public License. This except- */
/* ion does not  however invalidate  any other reasons  why the  executable */
/* file might be covered by the  GNU Public License.                        */
/*                                                                          */
/* GNAT was originally developed  by the GNAT team at  New York University. */
/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
/*                                                                          */
/****************************************************************************/

/* Routines to support runtime exception handling */

/* ??? We need this to define malloc on those machines that need it, but
   this is the wrong file when this is built for libgnat.a.  */

#include "config.h"
#include "a-ada.h"
#include "a-types.h"
#include "a-raise.h"
#include <stdio.h>

extern void (*system__tasking_soft_links__abort_defer) ();  
extern Exception_Id system__task_specific_data__get_gnat_exception ();
extern int system__task_specific_data__get_message_length ();
extern void system__task_specific_data__set_message_length (int);
extern char *system__task_specific_data__get_message_addr ();
extern int *system__task_specific_data__get_jmpbuf_address ();

extern struct Exception_Data constraint_error;
extern struct Exception_Data  program_error;
extern struct Exception_Data  storage_error;

/* procedure called when an exception is raised and no handler will take care 
   of, the name of the exception is printed as well as the message if any */
void
__gnat_unhandled_exception (except)
     Exception_Id except;
{
  int msg_len = system__task_specific_data__get_message_length ();
  char *msg;

  fputs ("\nraised ", stdout);

  if (except->Full_Name)
    fputs (except->Full_Name, stdout);

  /* this should never happen */
  else
    fputs ("unhandled exception", stdout);


  if (msg_len > 0) 
    {
      msg = system__task_specific_data__get_message_addr ();
      msg [msg_len] = 0;
      fputs (" : ", stdout);
      fputs (msg, stdout);
    }

  fputs ("\n", stdout);
}

/* procedure that actually raise an exception using the longjmp method */ 
void
__gnat_raise_nodefer_with_msg (except)
     Exception_Id except;
{
  int *ptr = system__task_specific_data__get_jmpbuf_address ();

  system__task_specific_data__set_gnat_exception (except);
  if (ptr)
#if defined (sparc) && defined (__svr4__)
    siglongjmp (ptr, 1);
#else
    longjmp (ptr, 1);
#endif

  else 
    {
      __gnat_unhandled_exception (except);
      exit (1);

    }
}

/* procedure called when an exception is raised with no message */
void
__gnat_raise_nodefer (except)
     Exception_Id except;
{
  system__task_specific_data__set_message_length (0);
  __gnat_raise_nodefer_with_msg (except);
}

/* procedure called when an exception is raised with no message and abort 
   must be defered */
void 
__gnat_raise (except)
     Exception_Id except;
{
  (*system__tasking_soft_links__abort_defer) ();
  __gnat_raise_nodefer (except);
}

/* procedure called when an exception is raised with a message and abort 
   must be defered */
void 
__gnat_raise_with_msg (except)
     Exception_Id except;
{
  (*system__tasking_soft_links__abort_defer) ();
  __gnat_raise_nodefer_with_msg (except);
}

void
__gnat_reraise (flag)
     int flag;
{
  Exception_Id except = system__task_specific_data__get_gnat_exception ();

  if (flag)
    __gnat_raise (except);
  else
    __gnat_raise_nodefer (except);
}

void
__gnat_raise_constraint_error ()
{
  __gnat_raise (&constraint_error);
}

void
__gnat_raise_program_error ()
{
  __gnat_raise (&program_error);
}

void *
__gnat_malloc (size)
     __SIZE_TYPE__ size;
{
  void *result;

  /* If size is negative, it means we have an overflow situation.  */
  if (size < 0)
    __gnat_raise (&storage_error);

  /* Change size from zero to non-zero. We still want a proper pointer
     for the zero case because pointers to zero length objects have to
     be distinct, but we can't just go ahead and allocate zero bytes,
     since some malloc's return zero for a zero argument */

  if (size == 0)
    size = 1;

  result = (char *) malloc (size);

  if (result == 0)
    __gnat_raise (&storage_error);

  return result;
}
