------------------------------------------------------------------------------
--                                                                          --
--                               PM Bindings                                --
--                                                                          --
--                                  GPI                                     --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                            $Revision: .17 $                              --
--
--  .11 
--       First version of GPI 
--       Char_String_At
--
--  .12  
--       Background_Mix_Type
--       Set_Background_Mix
--       Color_Type
--       Set_Background_Color
--       Set_Color
--
--  .13
--     Converted Text_Status_Type to Status_Type
--     GPI_Error exception       
--     Set_Current_Position
--     Line    
--     Box
--     Set_Arc_Parameters
--     Full_Arc
--
--  .14
--    Char_String
--    Move
--    Set_Default_Arc_Parameters
--    Query_Default_Arc_Parameters
--    Query_Arc_Parameters
--    Query_Current_Position
--    Query_Line_End
--    Set_Line_End
--    Query_Line_Join
--    Set_Line_Join
--    Erase
--    Destroy_Ps
--
--   .16
--   Moved Color_Type to Win package.
--
--   .17
--                                                                             
--   Create_Logical_Font (not working correctly)
--   Set_Char_Set        (not working correctly)
--   Save (GpiSavePS)
--   Restore (GpiRestorePS)
--                                                                             
--                                                                          
--   Copyright (c) 1994,1995 Dimensional Media Systems, All Rights Reserved --
--                                                                          --
--   The PM bindings are free software; you can redistribute them and/or    --
--   modify them under terms of the GNU General Public License as published --
--   by the Free Software Foundation; either version 2, or (at your         --
--   option) any later version.  The PM bindings are distributed in the     --
--   hope that they will be useful, but WITH OUT ANY WARRANTY; without even --
--   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR    --
--   PURPOSE.  See the GNU General Public License for more details.  You    --
--   should have received a copy of the GNU General Public License          --
--   distributed with The PM bindings; see file COPYING.  If not, write to  --
--   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
--   For more information about these PM bindings and their usage with GNAT --
--   you can contact Bill Yow at                                            --
--                                                                          --  
--      Dimensional Media Systems (DMS)                                     --
--      1522 Festival Dr.                                                   --
--      Houston TX, 77062                                                   --
--      Phone - (713) 486-6116                                              --
--      Email - byow@mci.com                                                --
--                                                                          --
------------------------------------------------------------------------------

with Pm_Types;
with Win;

package GPI is

   GPI_Error  : exception;

   type Ps_Id_Type is private;

   Bad_Ps_Id   : constant Ps_Id_Type;
   Most_Recent : constant Ps_Id_Type;

   type Status_Type is (Okay, Hits, Error);

   type Background_Mix_Type is (
       Bm_Error,
       Bm_Default,
       Bm_Or,
       Bm_Overpaint,
       Bm_Leave_Alone,
       Bm_Xor,
       Bm_And,
       Bm_Subtract,
       Bm_Mask_Src_Not,
       Bm_Zero,
       Bm_Not_Merge_Src,
       Bm_Not_Xor_Src,
       Bm_Invert,
       Bm_Merge_Src_Not,
       Bm_Not_Copy_Src,
       Bm_Merge_Not_Src,
       Bm_Not_Mask_Src,
       Bm_One,
       Bm_Src_Transparent,
       Bm_Dest_Transparent);

   function Set_Background_Mix (
              Ps  : Win.Ps_Type;
              Mix : Background_Mix_Type) return Boolean;
             
   procedure Set_Background_Mix (
              Ps  : Win.Ps_Type;
              Mix : Background_Mix_Type);

  ---------------------------------------------

   function Set_Background_Color (
                  Ps    : Win.Ps_Type;
                  Color : Win.Color_Type) return Boolean;
       
   procedure Set_Background_Color (
                  Ps    : Win.Ps_Type;
                  Color : Win.Color_Type);

  ---------------------------------------------

   function Set_Color (
                  Ps    : Win.Ps_Type;
                  Color : Win.Color_Type) return Boolean;
       
   procedure Set_Color (
                  Ps    : Win.Ps_Type;
                  Color : Win.Color_Type);

  ---------------------------------------------

   function Char_String_At (
                  Ps    : Win.Ps_Type;
                  Point : Win.Point_Type;
                  Text  : String) return Status_Type;

   procedure Char_String_At (
                  Ps    : in Win.Ps_Type;
                  Point : in Win.Point_Type;
                  Text  : in String);

  ---------------------------------------------

   function Char_String (
                  Ps    : Win.Ps_Type;
                  Text  : String) return Status_Type;

   procedure Char_String (
                  Ps    : in Win.Ps_Type;
                  Text  : in String);

  ---------------------------------------------

   subtype Str_8_Type is String (1 .. 8);

   type Local_Identifier_Type is range -1 .. 254;

   subtype LCID_Type is Local_Identifier_Type range 0 .. 254;

   LCID_Default : constant Local_Identifier_Type := 0;
   LCID_All     : constant Local_Identifier_Type := -1;
   LCID_Error   : constant Local_Identifier_Type := -1;

   type Font_Selections_Type is (
        Italic, Underscore, Outline, Strikeout, Bold);

   type Font_Selection_Type is array (Font_Selections_Type) of Boolean;

   type Font_Types is (Kerning, MBCS, DBCS, Anti_Aliased);

   type Font_Type is array (Font_Types) of Boolean;

   type Font_Usage_Types is (No_Mix, Use_Outline, Transformable);

   type Font_Usage_Type is array (Font_Usage_Types) of Boolean;

   Face_Size    : constant := 32;

   subtype Face_Name_Type is String (1 .. Face_Size);

   Unknown_Registry        : constant := 0;
   Use_Current_Code_Page   : constant := 0; 
   Outline_Font_Dimensions : constant := 0;

   type Font_Attributes_Type is
     record
       Selections             : Font_Selection_Type;
       Match                  : Pm_Types.Long;
       Face_Name              : Face_Name_Type;
       Registry               : Pm_Types.U_Short;
       Code_Page              : Pm_Types.U_Short;
       Max_Baseline_Extension : Win.Pixel_Type;
       Average_Char_Width     : Win.Pixel_Type;
       Type_of_Font           : Font_Type;
       Usage                  : Font_Usage_Type;
    end record;

   type Font_Status_Type is (
          Using_Default_Font, Font_Match);

   --Procedure can raise GPI_Error
   procedure Create_Logical_Font (
               Ps         : in     Win.Ps_Type;
               Name       : in     Str_8_Type; 
               Local_ID   : in     LCID_Type;
               Attributes : in     Font_Attributes_Type;
               Status     :    out Font_Status_Type);

  ---------------------------------------------

  procedure Set_Char_Set (
               Ps       : in Win.Ps_Type;
               Local_Id : in LCID_Type); 

  ---------------------------------------------

   function Set_Current_Position (
              Ps    : Win.Ps_Type;
              Point : Win.Point_Type) return Boolean;

   procedure Set_Current_Position (
              Ps    : Win.Ps_Type;
              Point : Win.Point_Type);

  ---------------------------------------------

   function Query_Current_Position (
              Ps    : Win.Ps_Type;
              Point : Win.Point_Pointer_Type) return Boolean;

   procedure Query_Current_Position (
              Ps    : in     Win.Ps_Type;
              Point :    out Win.Point_Type);

  ---------------------------------------------

   function Restore (Ps : Win.Ps_Type;
                     Ps_Id : Ps_Id_Type) return Boolean;

   procedure Restore (Ps : Win.Ps_Type;
                      Ps_ID : Ps_Id_Type);

  ---------------------------------------------

   function Save (Ps : Win.Ps_Type) return Ps_Id_Type;

  ---------------------------------------------

   function Erase (Ps : Win.Ps_Type) return Boolean;

   procedure Erase (Ps : Win.Ps_Type);

  ---------------------------------------------

   function Destroy_Ps (Ps : Win.Ps_Type) return Boolean;

   procedure Destroy_Ps (Ps : in Win.Ps_Type);

  ---------------------------------------------

   function Move (
              Ps    : Win.Ps_Type;
              Point : Win.Point_Type) return Boolean;

   procedure Move (
              Ps    : Win.Ps_Type;
              Point : Win.Point_Type);

  ---------------------------------------------

   type Line_End_Style_Type is (
         Line_End_Error,
         Line_End_Default,
         Line_End_Flat,
         Line_End_Square,
         Line_End_Round);

   type Line_Join_Style_Type is (
         Line_Join_Error,
         Line_Join_Default,
         Line_Join_Bevel,
         Line_Join_Round,
         Line_Join_Mitre);

  ---------------------------------------------

   function Set_Line_End (
              Ps        : Win.Ps_Type;
              Style     : Line_End_Style_Type) return Boolean;

   procedure Set_Line_End (
              Ps        : in Win.Ps_Type;
              Style     : in Line_End_Style_Type);

  ---------------------------------------------

   function Query_Line_End (Ps : Win.Ps_Type) return Line_End_Style_Type;

  ---------------------------------------------

   function Set_Line_Join (
              Ps        : Win.Ps_Type;
              Style     : Line_Join_Style_Type) return Boolean;

   procedure Set_Line_Join (
              Ps        : in Win.Ps_Type;
              Style     : in Line_Join_Style_Type);

  ---------------------------------------------

   function Query_Line_Join (Ps : Win.Ps_Type) return Line_Join_Style_Type;

  ---------------------------------------------

   function Line (
              Ps        : Win.Ps_Type;
              End_Point : Win.Point_Type) return Status_Type;

   procedure Line (
              Ps        : Win.Ps_Type;
              End_Point : Win.Point_Type);

   procedure Line (
              Ps          : Win.Ps_Type;
              Start_Point : Win.Point_Type;
              End_Point   : Win.Point_Type);

  ---------------------------------------------

   type Outline_Style_Type is (
      Dro_Fill,          --Fill the interior of the box without drawing 
                         --the outline.
      Dro_Outline,       --Draw the outline of the box.
      Dro_Outline_Fill); --Draw the outline of the box and fill the interior.
 
   function Box (
              Ps            : Win.Ps_Type;
              Corner_Point  : Win.Point_Type;
              Outline_Style : Outline_Style_Type;
              Horz_Rounding : Pm_Types.Long;
              Vert_Rounding : Pm_Types.Long) return Status_Type; 

   procedure Box (
              Ps            : in Win.Ps_Type;
              Corner_Point  : in Win.Point_Type;
              Outline_Style : in Outline_Style_Type;
              Horz_Rounding : in Pm_Types.Long;
              Vert_Rounding : in Pm_Types.Long);

   procedure Box (
              Ps            : in Win.Ps_Type;
              Start_Corner  : in Win.Point_Type;
              End_Corner    : in Win.Point_Type;
              Outline_Style : in Outline_Style_Type;
              Horz_Rounding : in Pm_Types.Long;
              Vert_Rounding : in Pm_Types.Long);

 ---------------------------------------------
   type Arc_Parameter_Type is
     record
       lP : Pm_Types.Long;
       lQ : Pm_Types.Long;
       lR : Pm_Types.Long;
       lS : Pm_Types.Long;
     end record;

   type Arc_Parameter_Pointer_Type is access Arc_Parameter_Type;

   function Set_Arc_Parameters (
              Ps         : Win.Ps_Type;
              Parameters : Arc_Parameter_Type) return Boolean;

   procedure Set_Arc_Parameters (
              Ps         : in Win.Ps_Type;
              Parameters : in Arc_Parameter_Type);

 ---------------------------------------------

   function Query_Arc_Parameters (
              Ps         : Win.Ps_Type;
              Parameters : Arc_Parameter_Pointer_Type) return Boolean;

   procedure Query_Arc_Parameters (
              Ps         : in     Win.Ps_Type;
              Parameters :    out Arc_Parameter_Type);

 ---------------------------------------------

   function Set_Default_Arc_Parameters (
              Ps         : Win.Ps_Type;
              Parameters : Arc_Parameter_Type) return Boolean;

   procedure Set_Default_Arc_Parameters (
              Ps         : in Win.Ps_Type;
              Parameters : in Arc_Parameter_Type);

 ---------------------------------------------

   function Query_Default_Arc_Parameters (
              Ps         : Win.Ps_Type;
              Parameters : Arc_Parameter_Pointer_Type) return Boolean;

   procedure Query_Default_Arc_Parameters (
              Ps         : in     Win.Ps_Type;
              Parameters :    out Arc_Parameter_Type);

 ---------------------------------------------

--   type Multipler_Type is delta 0.001 range 0.0 .. 255.0;
     type Multipler_Type is range 0 .. 255;  --Will be converted to fixed 
                                             --type when implement.

   function Full_Arc (
              Ps            : Win.Ps_Type;
              Outline_Style : Outline_Style_Type;
              Multiplier    : Multipler_Type) return Status_Type;

   procedure Full_Arc (
              Ps            : in Win.Ps_Type;
              Outline_Style : in Outline_Style_Type;
              Multiplier    : in Multipler_Type);

   procedure Full_Arc (
              Ps            : in Win.Ps_Type;
              Center        : in Win.Point_Type;
              Arc_Params    : in Arc_Parameter_Type;
              Outline_Style : in Outline_Style_Type;
              Multiplier    : in Multipler_Type);

 ---------------------------------------------

  private 

   type Ps_Id_Type is new Pm_Types.Long;

   Bad_Ps_Id   : constant Ps_Id_Type :=  0;
   Most_Recent : constant Ps_Id_Type := -1;


end GPI;
