------------------------------------------------------------------------------
--                                                                          --
--                               PM Bindings                                --
--                                                                          --
--                          Pipes_and_Files_Test                            --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                            $Revision: .17 $                              --
--                                                                          --
--     Copyright (c) 1994, 1995 Dimensional Media Systems, All Rights       --
--     Reserved                                                             --
--                                                                          --
--   The PM bindings are free software; you can redistribute them and/or    --
--   modify them under terms of the GNU General Public License as published --
--   by the Free Software Foundation; either version 2, or (at your         --
--   option) any later version.  The PM bindings are distributed in the     --
--   hope that they will be useful, but WITH OUT ANY WARRANTY; without even --
--   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR    --
--   PURPOSE.  See the GNU General Public License for more details.  You    --
--   should have received a copy of the GNU General Public License          --
--   distributed with The PM bindings; see file COPYING.  If not, write to  --
--   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
--   For more information about these PM bindings and their usage with GNAT --
--   you can contact Bill Yow at                                            --
--                                                                          --  
--      Dimensional Media Systems (DMS)                                     --
--      1522 Festival Dr.                                                   --
--      Houston TX, 77062                                                   --
--      Phone - (713) 486-6116                                              --
--      Email - byow@mci.com                                                --
--                                                                          --
------------------------------------------------------------------------------

with Dos;
with Pm_Types;

with Text_Io;


procedure Pipes_And_Files_Test is

   package Pf  renames Dos;
   package Tio renames Text_Io;

   subtype Handle is Pf.File_Handle;


   Orig_Stdout : Pf.File_Handle := Pf.Empty_Handle;
   Stdout_Redi : Pf.File_Handle := 16#0001#;
   Read, Write : Pf.Pipe_Handle := Pf.Empty_Handle;

   Return_Code : Dos.API_Return_Code;

   Name  : Dos.Object_Buffer_Type (1..124);
   Codes : Dos.Result_Codes_Type;
   Empty : String (1..0);

   type Buffer is new String (1..50);

   Data   : Buffer := (others => ' ');
   Length : Natural:= 10;

   procedure Read_String is new Pf.Read (Buffer);


   function To_String (Item : Dos.API_Return_Code) return String is
      subtype API_Code is Dos.API_Return_Code;
   begin
      return API_Code'IMAGE (Item);
   end To_String;

begin

   Pf.Duplicate_Handle (Pf.Standard_Output, Orig_Stdout, Return_Code);
   Tio.Put_Line ("Duplicate Handle Return Code: " & To_String (Return_Code));

   Pf.Create_Pipe (256, Write, Read, Return_Code);
   Tio.Put_Line ("Create Pipe Return Code: " & To_String (Return_Code));
   Tio.Put_Line ("  Read:  " & Handle'IMAGE (Read));
   Tio.Put_Line ("  Write: " & Handle'IMAGE (Write));

   Pf.Duplicate_Handle (Write, Stdout_Redi, Return_Code);
   Tio.Put_Line ("Duplicate Handle Return Code: " & To_String (Return_Code));

   Return_Code := Dos.Exec_Program (Name, Dos.Exec_Async, Empty, Empty, Codes,
                                    "hello_world");
   Tio.Put_Line ("Exec Program Return Code: " & To_String (Return_Code));

   Pf.Close (Write, Return_Code);
   Tio.Put_Line ("Close Return Code: " & To_String (Return_Code));

   Pf.Duplicate_Handle (Orig_Stdout, Stdout_Redi, Return_Code);
   Tio.Put_Line ("Duplicate Handle Return Code: " & To_String (Return_Code));

   Read_String (Read, Data, Length, Return_Code);
   Tio.Put_Line ("Read String Return Code: " & To_String (Return_Code));

   Tio.New_Line (3);
   Tio.Put_Line (String (Data));
   Tio.Put_Line ("Bytes Read: " & Natural'IMAGE (Length));
   Tio.New_Line (3);

   Pf.Close (Read, Return_Code);
   Tio.Put_Line ("Exiting Close Return Code: " & To_String (Return_Code));

end Pipes_And_Files_Test;
