------------------------------------------------------------------------------
--                                                                          --
--                               PM Bindings                                --
--                                                                          --
--                                Port_Test                                 --
--                                                                          --
--                    Test of the OS/2 Port_IO routines                     -- 
--                                                                          --
--                            $Revision: .17 $                              --
--                                                                          --
--     Copyright (c) 1995 Dimensional Media Systems, All Rights Reserved    --
--                                                                          --
--   The OS/2 bindings EMX are free software; you can redistribute them and/or --
--   modify them under terms of the GNU General Public License as published --
--   by the Free Software Foundation; either version 2, or (at your         --
--   option) any later version.  The PM bindings are distributed in the     --
--   hope that they will be useful, but WITH OUT ANY WARRANTY; without even --
--   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR    --
--   PURPOSE.  See the GNU General Public License for more details.  You    --
--   should have received a copy of the GNU General Public License          --
--   distributed with The PM bindings; see file COPYING.  If not, write to  --
--   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
--   For more information about these PM bindings and their usage with GNAT --
--   you can contact Bill Yow at                                            --
--                                                                          --  
--      Dimensional Media Systems (DMS)                                     --
--      1522 Festival Dr.                                                   --
--      Houston TX, 77062                                                   --
--      Phone - (713) 486-6116                                              --
--      Email - Byow@mci.com                                                --
--                                                                          --
------------------------------------------------------------------------------

with Pm_Types;
with Port_Io;
with Text_Io;

procedure Port_Test is

   Output_Port  : constant  := 16#0378#;  --Parallel port LPT1
   Status_Port  : constant  := Output_Port + 1;
   Control_Port : constant  := Output_Port + 2;

   Ready_Bit    : constant  := 16#80#;
   Strobe_Out   : constant  := 16#0D#;
   Strobe_Off   : constant  := 16#0C#;

   type Byte is mod 256;

   ----------------------------------------

   procedure Send_Char (Char : Character) is
      Count  : Integer := 0;
      Status : Pm_Types.U_Byte;

      Value  : Byte;

      use Pm_Types;
     begin

      loop

        Status := Port_Io.In_Port (Status_Port);

        Value := Byte (Status) and Byte (Ready_Bit);

        exit when Value /= 0;

        if Count > 10_000 then
           Text_Io.Put_Line ("Still Busy");
           Count := 0;
        end if;

      end loop;

      Port_Io.Out_Port (Output_Port, Pm_Types.U_Byte (Character'Pos (Char)));
      Port_Io.Out_Port (Control_Port, Strobe_Out);
      Port_Io.Out_Port (Control_Port, Strobe_Off);
       
    end Send_Char;     

  ---------------------------------

   procedure Send_Message (Str : String) is
     begin
      for I in Str'range loop
        Send_Char (Str (I));
      end loop;
    end Send_Message;

  begin

    Text_Io.Put_Line ("Senting up port access");

    Port_Io.Access_Ports (Output_Port, Control_Port);

    Text_Io.Put_Line ("Sending Message 1");

    Send_Message ("This is the first message." & ASCII.LF);        

    Text_Io.Put_Line ("Sending Message 2");
    Send_Message ("This is the Second message." & ASCII.LF);        

    Text_Io.Put_Line ("Sending Message 3");
    Send_Message ("This is the Third message." & ASCII.LF);        
     
    Text_Io.Put_Line ("Sending Page feed");
    Send_Message ("  " & ASCII.FF);

  end Port_Test;